from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any

import torch
import torch.nn.functional as F
from packaging.version import Version
from tqdm import tqdm
from transformers import AutoModel, AutoTokenizer
from transformers import __version__ as transformers_version

from mteb._requires_package import requires_package
from mteb.abstasks.task_metadata import TaskMetadata
from mteb.models import CrossEncoderWrapper
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.types import PromptType

if TYPE_CHECKING:
    from collections.abc import Callable

    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput

logger = logging.getLogger(__name__)

NV_RETRIEVER_CITATION = """@misc{lee2025nvembedimprovedtechniquestraining,
      title={NV-Embed: Improved Techniques for Training LLMs as Generalist Embedding Models},
      author={Chankyu Lee and Rajarshi Roy and Mengyao Xu and Jonathan Raiman and Mohammad Shoeybi and Bryan Catanzaro and Wei Ping},
      year={2025},
      eprint={2405.17428},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2405.17428},
}"""

LlamaEmbedNemotron_CITATION = """@misc{babakhin2025llamaembednemotron8buniversaltextembedding,
      title={Llama-Embed-Nemotron-8B: A Universal Text Embedding Model for Multilingual and Cross-Lingual Tasks},
      author={Yauhen Babakhin and Radek Osmulski and Ronay Ak and Gabriel Moreira and Mengyao Xu and Benedikt Schifferer and Bo Liu and Even Oldridge},
      year={2025},
      eprint={2511.07025},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2511.07025},
}"""


def _instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    return f"Instruct: {instruction}\nQuery: " if instruction else ""


nvidia_training_datasets = {
    # source: https://arxiv.org/pdf/2405.17428
    "ArguAna",
    "ArguAna-PL",
    "ArguAna-NL",  # translation not trained on
    "NanoArguAnaRetrieval",
    "HotpotQA",
    "HotpotQA-PL",  # translation not trained on
    "HotpotQA-NL",  # translation not trained on
    "HotpotQAHardNegatives",
    "MSMARCO",
    "MSMARCOHardNegatives",
    "NanoMSMARCORetrieval",
    "MSMARCO-PL",  # translation not trained on
    "mMARCO-NL",  # translation not trained on
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    "NQ-NL",  # translation not trained on
    "FEVER",
    "FEVER-NL",  # translation not trained on
    "FEVERHardNegatives",
    "NanoFEVERRetrieval",
    "FiQA2018",
    "FiQA2018-NL",  # translation not trained on
    "STS12",
    "STS22",
    "AmazonReviewsClassification",
    "AmazonCounterfactualClassification",
    "Banking77Classification",
    "EmotionClassification",
    "ImdbClassification",
    "MTOPIntentClassification",
    "ToxicConversationsClassification",
    "TweetSentimentExtractionClassification",
    "ArxivClusteringP2P",
    "ArxivClusteringP2P.v2",
    "ArxivClusteringS2S",
    "BiorxivClusteringP2P",
    "BiorxivClusteringP2P.v2",
    "BiorxivClusteringS2S",
    "BiorxivClusteringS2S.v2",
    "MedrxivClusteringP2P",
    "MedrxivClusteringP2P.v2",
    "MedrxivClusteringS2S",
    "MedrxivClusteringS2S.v2",
    "TwentyNewsgroupsClustering",
    "TwentyNewsgroupsClustering.v2",
    "StackExchangeClustering",
    "StackExchangeClustering.v2",
    "StackExchangeClusteringP2P",
    "StackExchangeClusteringP2P.v2",
    "RedditClustering",
    "RedditClustering.v2",
    "RedditClusteringP2P",
    "RedditClusteringP2P.v2",
    "STSBenchmark",
    "STSBenchmarkMultilingualSTS",  # translated, not trained on
    "MIRACLRetrieval",
    "MIRACLRetrievalHardNegatives",
    "MIRACLReranking",
    "MrTidyRetrieval",
}


class _NVEmbedWrapper(InstructSentenceTransformerModel):
    """Inherited, because nvembed requires `sbert==2`, but it doesn't have tokenizers kwargs"""

    def __init__(
        self,
        model_name: str,
        revision: str,
        instruction_template: str
        | Callable[[str, PromptType | None], str]
        | None = None,
        max_seq_length: int | None = None,
        apply_instruction_to_passages: bool = True,
        padding_side: str | None = None,
        add_eos_token: bool = False,
        prompts_dict: dict[str, str] | None = None,
        **kwargs: Any,
    ):
        from sentence_transformers import __version__ as sbert_version

        required_transformers_version = "4.42.4"
        required_sbert_version = "2.7.0"

        if Version(transformers_version) != Version(required_transformers_version):
            raise RuntimeError(
                f"transformers version {transformers_version} is not match with required "
                f"install version {required_transformers_version} to run `nvidia/NV-Embed-v2`"
            )

        if Version(sbert_version) != Version(required_sbert_version):
            raise RuntimeError(
                f"sbert version {sbert_version} is not match with required "
                f"install version {required_sbert_version} to run `nvidia/NV-Embed-v2`"
            )

        requires_package(
            self, "flash_attn", model_name, "pip install 'mteb[flash_attention]'"
        )

        from sentence_transformers import SentenceTransformer

        if (
            isinstance(instruction_template, str)
            and "{instruction}" not in instruction_template
        ):
            raise ValueError(
                "Instruction template must contain the string '{instruction}'."
            )
        if instruction_template is None:
            logger.warning(
                "No instruction template provided. Instructions will be used as-is."
            )

        self.instruction_template = instruction_template

        self.model_name = model_name
        self.model = SentenceTransformer(model_name, revision=revision, **kwargs)
        self.model.tokenizer.padding_side = padding_side
        self.model.tokenizer.add_eos_token = add_eos_token

        if max_seq_length:
            # https://github.com/huggingface/sentence-transformers/issues/3575
            self.model.max_seq_length = max_seq_length
        self.apply_instruction_to_passages = apply_instruction_to_passages
        self.prompts_dict = prompts_dict


NV_embed_v2 = ModelMeta(
    loader=_NVEmbedWrapper,
    loader_kwargs=dict(
        instruction_template=_instruction_template,
        trust_remote_code=True,
        max_seq_length=32768,
        padding_side="right",
        # for nv-embed, we add eos token to each input example
        add_eos_token=True,
    ),
    name="nvidia/NV-Embed-v2",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="7604d305b621f14095a1aa23d351674c2859553a",
    release_date="2024-09-09",  # initial commit of hf model.
    n_parameters=7_850_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=14975,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    max_tokens=32768,
    reference="https://huggingface.co/nvidia/NV-Embed-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=True,
    training_datasets=nvidia_training_datasets,
    public_training_code=None,
    public_training_data=None,
    citation=NV_RETRIEVER_CITATION,
)

NV_embed_v1 = ModelMeta(
    loader=_NVEmbedWrapper,
    loader_kwargs=dict(
        instruction_template=_instruction_template,
        trust_remote_code=True,
        max_seq_length=32768,
        padding_side="right",
        # for nv-embed, we add eos token to each input example
        add_eos_token=True,
    ),
    name="nvidia/NV-Embed-v1",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="570834afd5fef5bf3a3c2311a2b6e0a66f6f4f2c",
    release_date="2024-09-13",  # initial commit of hf model.
    n_parameters=7_850_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=14975,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    max_tokens=32768,
    reference="https://huggingface.co/nvidia/NV-Embed-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    training_datasets=nvidia_training_datasets,
    public_training_code=None,
    public_training_data=None,
    citation=NV_RETRIEVER_CITATION,
)

llama_embed_nemotron_evaluated_languages = [
    "afr-Latn",
    "amh-Ethi",
    "ara-Arab",
    "arq-Arab",
    "ary-Arab",
    "bbc-Latn",
    "ben-Beng",
    "bug-Latn",
    "bul-Cyrl",
    "cat-Latn",
    "ces-Latn",
    "cmn-Hans",
    "dan-Latn",
    "deu-Latn",
    "ell-Grek",
    "eng-Latn",
    "est-Latn",
    "fas-Arab",
    "fin-Latn",
    "fra-Latn",
    "hau-Latn",
    "hin-Deva",
    "hun-Latn",
    "hye-Armn",
    "ibo-Latn",
    "ind-Latn",
    "ita-Latn",
    "jav-Latn",
    "jpn-Jpan",
    "kin-Latn",
    "kor-Hang",
    "kor-Kore",
    "lav-Latn",
    "lin-Latn",
    "lug-Latn",
    "mad-Latn",
    "min-Latn",
    "mlt-Latn",
    "nld-Latn",
    "nob-Latn",
    "nor-Latn",
    "orm-Ethi",
    "pcm-Latn",
    "pol-Latn",
    "por-Latn",
    "ron-Latn",
    "run-Latn",
    "rus-Cyrl",
    "slk-Latn",
    "slv-Latn",
    "sna-Latn",
    "som-Latn",
    "spa-Latn",
    "sqi-Latn",
    "srp-Cyrl",
    "sun-Latn",
    "swa-Latn",
    "swe-Latn",
    "tel-Telu",
    "tha-Thai",
    "tir-Ethi",
    "tur-Latn",
    "vie-Latn",
    "xho-Latn",
    "yor-Latn",
    "zho-Hans",
]

TASK_PROMPTS = {
    # Classification
    "BulgarianStoreReviewSentimentClassfication": "Classify user reviews into positive or negative sentiment",
    "CzechProductReviewSentimentClassification": "Classify product reviews into positive or negative sentiment",
    "GreekLegalCodeClassification": "Given a greek legal text, classify its topic",
    "DBpediaClassification": "Given a Wikipedia articles, categorized it into classes based on its DBpedia ontology",
    "FinancialPhrasebankClassification": "Given financial news, categorized by sentiment into positive, negative, or neutral",
    "PoemSentimentClassification": "Given a poem, categorized by sentiment into positive, no_impact, negative or mixed",
    "TweetTopicSingleClassification": "Given a twitter, classify its topic",
    "EstonianValenceClassification": "Given a news article, categorized by sentiment into negatiivne, positiivne, neutraalne or vastuolulin",
    "FilipinoShopeeReviewsClassification": "Given a shop review, classify its rating on a scale from 1 to 5",
    "GujaratiNewsClassification": "Given a Gujarati news articles, classify its topic",
    "SentimentAnalysisHindi": "Given a hindi text, categorized by sentiment into positive, negative or neutral",
    "IndonesianIdClickbaitClassification": "Given an Indonesian news headlines, classify its into clickbait or non-clickbait",
    "ItaCaseholdClassification": "Given a judgments, classify its topic",
    "KorSarcasmClassification": "Given a twitter, categorized it into sarcasm or not_sarcasm",
    "KurdishSentimentClassification": "Given a text, categorized by sentiment into positive or negative",
    "MacedonianTweetSentimentClassification": "Given a Macedonian tweet, categorized by sentiment into positive, negative, or neutral",
    "AfriSentiClassification": "Given a text, categorized by sentiment into positive, negative, or neutral",
    "CataloniaTweetClassification": "Given a tweet, categorized by sentiment into AGAINST, FAVOR or NEUTRAL",
    "CyrillicTurkicLangClassification": "Given a text, classify its language",
    "IndicLangClassification": "Given a text, classify its language",
    "MasakhaNEWSClassification": "Classify the News in the given texts into one of the seven category: politics,sports,health,business,entertainment,technology,religion ",
    "MultiHateClassification": "Given a text, categorized by sentiment into hate or non-hate",
    "NusaParagraphEmotionClassification": "Given a paragraph, classify its emotion",
    "NusaX-senti": "Given a text, categorized by sentiment into positive or negative",
    "SwissJudgementClassification": "Given a news article, categorized it into approval or dismissal",
    "NepaliNewsClassification": "Given a news article, categorized it into business, entertainment or sports",
    "OdiaNewsClassification": "Given a news article, categorized it into business, entertainment or sports",
    "PunjabiNewsClassification": "Given a news article, categorized it into two-classes",
    "PolEmo2.0-OUT": "Classify the sentiment of out-of-domain (products and school) online reviews",
    "PAC": 'Classify the sentence into one of the two types: "BEZPIECZNE_POSTANOWIENIE_UMOWNE" and "KLAUZULA_ABUZYWNA"',
    "SinhalaNewsClassification": "Given a news article, categorized it into political, business, technology, sports and Entertainment",
    "CSFDSKMovieReviewSentimentClassification": "Given a movie review, classify its rating on a scale from 0 to 5",
    "SiswatiNewsClassification": "Given a news article, classify its topic",
    "SlovakMovieReviewSentimentClassification": "Given a movie review, categorized it into positive or negative",
    "SwahiliNewsClassification": "Given a news article, classify its domain",
    "TswanaNewsClassification": "Given a news article, classify its topic",
    "IsiZuluNewsClassification": "Given a news article, classify its topic",
    # Clustering
    "WikiCitiesClustering": "Identify of Wikipedia articles of cities by country",
    "MasakhaNEWSClusteringS2S": "Identify the topic or theme of the given news articles based on the titles",
    "RomaniBibleClustering": "Identify verses from the Bible in Kalderash Romani by book.",
    "ArXivHierarchicalClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
    "ArXivHierarchicalClusteringS2S": "Identify the main and secondary category of Arxiv papers based on the titles",
    "BigPatentClustering.v2": "Identify the category of documents from the Big Patent dataset",
    "AlloProfClusteringS2S.v2": "Identify the topic of document titles from Allo Prof dataset",
    "HALClusteringS2S.v2": "Identify the topic of titles from HAL",
    "SIB200ClusteringS2S": "Identify the category of documents",
    "WikiClusteringP2P.v2": "Identify the category of wiki passages",
    "PlscClusteringP2P.v2": "Identify the category of titles+abstracts from Library of Science",
    # Multilabel Classification
    "KorHateSpeechMLClassification": "Given a Korean online news comments, classify its fine-grained hate speech classes",
    "MalteseNewsClassification": "Given a maltese new, classify its topic",
    "MultiEURLEXMultilabelClassification": "Given a text, classify its topic",
    "BrazilianToxicTweetsClassification": "Given a tweet, classify its topic",
    # Retrieval
    "StackOverflowQA": "Given a question about coding, retrieval code or passage that can solve user's question",
    "AILAStatutes": "Identifying the most relevant statutes for a given situation",
    "ArguAna": {
        "query": "Given a claim, find documents that refute the claim",
        "document": "Given a claim, find documents that refute the claim",
    },
    "HagridRetrieval": "Given a question, retrieve passages that answer the question",
    "LegalBenchCorporateLobbying": "Given a question, retrieve passages that answer the question",
    "LEMBPasskeyRetrieval": "Given a question, retrieve passages that answer the question",
    "BelebeleRetrieval": "Given a question, retrieve passages that answer the question",
    "MLQARetrieval": "Given a question, retrieve passages that answer the question",
    "StatcanDialogueDatasetRetrieval": "Given a question, retrieve passages that answer the question",
    "WikipediaRetrievalMultilingual": "Given a question, retrieve passages that answer the question",
    "Core17InstructionRetrieval": "Given a question, retrieve passages that answer the question",
    "News21InstructionRetrieval": "Given a question, retrieve passages that answer the question",
    "Robust04InstructionRetrieval": "Given a question, retrieve passages that answer the question",
    "MIRACLRetrievalHardNegatives": "Given a question, retrieve passages that answer the question",
    # Reranking
    "WebLINXCandidatesReranking": "Given a question, retrieve passages that answer the question",
    "AlloprofReranking": "Given a question, retrieve passages that answer the question",
    "WikipediaRerankingMultilingual": "Given a question, retrieve passages that answer the question",
}

llama_embed_nemotron_training_datasets = {
    "AmazonReviewsClassification",
    "BioASQ",
    "EmotionClassification",
    "FEVER",
    "GooAQ",
    "HotpotQA",
    "MIRACLRetrieval",
    "MSMARCO",
    "MrTidyRetrieval",
    "Nemotron-CC-v2",
    "NFCorpus",
    "NQ",
    "PAQ",
    "RedditClustering.v2",
    "RedditClusteringP2P.v2",
    "SQuAD",
    "TriviaQA",
}


class LlamaEmbedNemotron(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str | None = None,
    ) -> None:
        required_transformers_version = "4.51.0"
        if Version(transformers_version) != Version(required_transformers_version):
            raise ImportError(
                f"{model_name} requires transformers library version {required_transformers_version}, but it was not found in your environment. "
                + f"If you want to load {model_name} model, please run `pip install 'mteb[llama-embed-nemotron]'` to install the required package."
            )

        requires_package(
            self, "flash_attn", model_name, "pip install 'mteb[flash_attention]'"
        )

        self.model_name = model_name
        self.revision = revision
        self.max_seq_length = 4096
        self.attn_implementation = (
            "flash_attention_2" if torch.cuda.is_available() else "eager"
        )
        self.device = device or ("cuda" if torch.cuda.is_available() else "cpu")
        self.task_prompts = TASK_PROMPTS
        self.instruction_template = self._instruction_template

        self.tokenizer = AutoTokenizer.from_pretrained(
            self.model_name,
            trust_remote_code=True,
            padding_side="left",
            revision=self.revision,
        )

        self.model = AutoModel.from_pretrained(
            self.model_name,
            trust_remote_code=True,
            torch_dtype=torch.float16,
            attn_implementation=self.attn_implementation,
            revision=self.revision,
        ).eval()

        self.model = self.model.to(self.device)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        """Encode sentences with task-specific instructions."""

        instruction = self._get_task_specific_instruction(task_metadata, prompt_type)

        prefix = self.format_instruction(instruction, prompt_type)
        return self._extract_embeddings(inputs, instruction=prefix, **kwargs)

    def _get_task_specific_instruction(
        self,
        task_metadata: TaskMetadata,
        prompt_type: PromptType | None,
    ) -> str:
        """Get instruction for a specific task, applying task-specific overrides."""
        # First, get the base instruction using custom prompts or task metadata
        instruction = self._get_base_instruction(task_metadata, prompt_type)

        # Apply task-type-specific overrides
        instruction = self._apply_task_type_overrides(
            instruction, task_metadata.type, task_metadata.name, prompt_type
        )

        return instruction

    def _get_base_instruction(
        self, task_metadata: TaskMetadata, prompt_type: PromptType | None
    ) -> str:
        """Get the base instruction from task-specific prompts or task metadata."""
        task_name = task_metadata.name

        # Check if task has custom prompt in TASK_PROMPTS
        if task_name in self.task_prompts:
            instruction = self.task_prompts[task_name]
            # Handle dict-based prompts (e.g., ArguAna with different query/document prompts)
            if isinstance(instruction, dict) and prompt_type:
                return instruction.get(prompt_type.value, "")
            return instruction if isinstance(instruction, str) else ""

        # Fall back to AbsEncoder's get_instruction method for task metadata
        return self.get_instruction(task_metadata, prompt_type)

    def _apply_task_type_overrides(
        self,
        instruction: str,
        task_type: str,
        task_name: str,
        prompt_type: PromptType | None,
    ) -> str:
        """Apply task-type-specific instruction overrides."""

        # For retrieval/reranking tasks, skip instruction for documents
        # unless it's a symmetric task (where both query and document use prompts)
        is_symmetric_task = task_name in self.task_prompts and isinstance(
            self.task_prompts[task_name], dict
        )

        if (
            ("Retrieval" in task_type or "Reranking" in task_type)
            and not is_symmetric_task
            and prompt_type == PromptType.document
        ):
            return ""

        # Override for STS and PairClassification tasks
        if task_type in ["STS", "PairClassification"]:
            return "Retrieve semantically similar text"

        # Override for BitextMining tasks
        if task_type in ["BitextMining"]:
            return "Retrieve parallel sentences"

        return instruction

    @staticmethod
    def _instruction_template(
        instruction: str, prompt_type: PromptType | None = None
    ) -> str:
        """Format instruction with the model-specific template."""

        return f"Instruct: {instruction}\nQuery: " if instruction else ""

    @staticmethod
    def average_pooling(
        last_hidden_states: torch.Tensor,
        attention_mask: torch.Tensor,
    ) -> torch.Tensor:
        last_hidden = last_hidden_states.masked_fill(
            ~attention_mask[..., None].bool(), 0.0
        )

        return last_hidden.sum(dim=1) / attention_mask.sum(dim=1)[..., None]

    def _extract_embeddings(
        self,
        dataloader: DataLoader[BatchedInput],
        instruction: str,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        all_embeddings = []
        for batch in tqdm(
            dataloader, desc="Extracting embeddings...", disable=not show_progress_bar
        ):
            with torch.inference_mode():
                inputs = self.tokenizer(
                    [instruction + t for t in batch["text"]],
                    max_length=self.max_seq_length,
                    truncation="longest_first",
                    padding=True,
                    return_tensors="pt",
                ).to(self.device)

                outputs = self.model(**inputs)
                if outputs.last_hidden_state.dtype == torch.float16:
                    outputs.last_hidden_state = outputs.last_hidden_state.to(
                        torch.float32
                    )

                embeddings = self.average_pooling(
                    last_hidden_states=outputs.last_hidden_state,
                    attention_mask=inputs["attention_mask"],
                )

                embeddings = F.normalize(embeddings, dim=-1)

            all_embeddings.append(embeddings.contiguous())

        result = (
            torch.cat(all_embeddings, dim=0)
            .detach()
            .cpu()
            .numpy()
            .astype("float32")[: len(dataloader.dataset)]
        )

        del all_embeddings
        if torch.cuda.is_available():
            torch.cuda.empty_cache()

        return result


llama_embed_nemotron_8b = ModelMeta(
    loader=LlamaEmbedNemotron,
    name="nvidia/llama-embed-nemotron-8b",
    model_type=["dense"],
    languages=llama_embed_nemotron_evaluated_languages,
    open_weights=True,
    revision="84a375593d27d3528beb4e104822515659e093b4",
    release_date="2025-10-23",
    n_parameters=7_504_924_672,
    n_embedding_parameters=None,
    memory_usage_mb=28629,
    embed_dim=4096,
    license="https://huggingface.co/nvidia/llama-embed-nemotron-8b/blob/main/LICENSE",
    max_tokens=32768,
    reference="https://huggingface.co/nvidia/llama-embed-nemotron-8b",
    similarity_fn_name="cosine",
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    use_instructions=True,
    training_datasets=llama_embed_nemotron_training_datasets,
    public_training_code="https://github.com/NVIDIA-NeMo/Automodel/tree/main/examples/biencoder/llama_embed_nemotron_8b",
    public_training_data="https://huggingface.co/datasets/nvidia/embed-nemotron-dataset-v1",
    contacts=["ybabakhin"],
    citation=LlamaEmbedNemotron_CITATION,
)


def _nemotron_rerank_model(model: str, revision: str, **kwargs) -> CrossEncoderWrapper:
    required_transformers_version = "4.47.1"

    if Version(transformers_version) != Version(required_transformers_version):
        raise RuntimeError(
            f"transformers version {transformers_version} is not match with required "
            f"install version {required_transformers_version} to run `nvidia/llama-nemotron-rerank-1b-v2`"
        )

    return CrossEncoderWrapper(
        model=model,
        revision=revision,
        **kwargs,
    )


nemotron_rerank_1b_v2 = ModelMeta(
    loader=_nemotron_rerank_model,
    loader_kwargs=dict(
        trust_remote_code=True,
        query_prefix="question:",
        passage_prefix=" \n \n passage:",
        model_kwargs={"torch_dtype": torch.float32},
    ),
    name="nvidia/llama-nemotron-rerank-1b-v2",
    revision="78efcfdc23b53a753f6c73f2d78b18132a34ac4d",
    release_date="2025-10-16",
    languages=["eng-Latn"],
    n_parameters=1235816448,
    memory_usage_mb=2357.0,
    max_tokens=4096,
    embed_dim=2048,
    license="https://www.nvidia.com/en-us/agreements/enterprise-software/nvidia-open-model-license/",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch", "Sentence Transformers"],
    reference="https://huggingface.co/nvidia/llama-nemotron-rerank-1b-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=None,
    training_datasets=set(
        # private
    ),
    adapted_from="meta-llama/Llama-3.2-1B",
    superseded_by=None,
    modalities=["text"],
    model_type=["cross-encoder"],
    citation=None,
    contacts=None,
)



from __future__ import annotations

import itertools
import logging
from collections import defaultdict
from typing import TYPE_CHECKING, Any, TypedDict

import numpy as np
from datasets import DatasetDict
from sklearn.base import clone
from sklearn.metrics import f1_score, label_ranking_average_precision_score
from sklearn.multioutput import MultiOutputClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.preprocessing import MultiLabelBinarizer
from typing_extensions import override

from mteb._create_dataloaders import create_dataloader
from mteb._evaluators.classification_metrics import hamming_score
from mteb.models import EncoderProtocol

from .classification import AbsTaskClassification

if TYPE_CHECKING:
    from pathlib import Path

    from numpy.typing import NDArray

    from mteb._evaluators.sklearn_evaluator import SklearnModelProtocol
    from mteb.models import MTEBModels
    from mteb.types import Array, EncodeKwargs

logger = logging.getLogger(__name__)


def _evaluate_classifier(
    embeddings_train: Array,
    y_train: NDArray[np.integer],
    embeddings_test: Array,
    classifier: SklearnModelProtocol,
) -> tuple[NDArray[np.integer | np.floating], SklearnModelProtocol]:
    classifier_copy: SklearnModelProtocol = clone(classifier)
    classifier_copy.fit(embeddings_train, y_train)
    return classifier_copy.predict(embeddings_test), classifier_copy


class MultilabelClassificationMetrics(TypedDict):
    """Scores for multilabel classification tasks.

    Attributes:
        accuracy: Accuracy of the classifier.
        lrap: Label Ranking Average Precision (LRAP) score.
        f1: Macro F1 score.
        hamming: Hamming score (label-based accuracy).
    """

    accuracy: float
    lrap: float
    f1: float
    hamming: float


class FullMultilabelClassificationMetrics(MultilabelClassificationMetrics):
    """Extended scores for multilabel classification tasks.

    Attributes:
        scores_per_experiment: List of individual experiment scores.
    """

    scores_per_experiment: list[MultilabelClassificationMetrics]


class AbsTaskMultilabelClassification(AbsTaskClassification):
    """Abstract class for multioutput classification tasks

    Attributes:
        dataset: Huggingface dataset containing the data for the task. Dataset must contain columns specified by input_column_name and label_column_name.
            Input column must contain the text or image to be classified, and label column must contain a list of labels for each example.
        input_column_name: Name of the column containing the input text.
        label_column_name: Name of the column containing the labels.
        samples_per_label: Number of samples to use pr. label. These samples are embedded and a classifier is fit using the labels and samples.
        evaluator_model: Classifier to use for evaluation. Must implement the SklearnModelProtocol.
    """

    evaluator_model: SklearnModelProtocol = KNeighborsClassifier(n_neighbors=5)
    input_column_name: str = "text"
    label_column_name: str = "label"

    @override
    def _evaluate_subset(  # type: ignore[override]
        self,
        model: MTEBModels,
        data_split: DatasetDict,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int = 1,
        **kwargs: Any,
    ) -> FullMultilabelClassificationMetrics:
        if not isinstance(model, EncoderProtocol):
            raise TypeError("Expected model to be an instance of EncoderProtocol")

        if isinstance(data_split, DatasetDict):
            data_split = data_split.select_columns(
                [self.input_column_name, self.label_column_name]
            )
        train_split = data_split[self.train_split]
        eval_split = data_split[hf_split]

        logger.info(
            "Running multilabel classification task - Sampling training data..."
        )
        scores = []
        # Bootstrap sample indices from training set for each experiment
        train_samples = []
        for _ in range(self.n_experiments):
            sample_indices, _ = self._undersample_data_indices(
                train_split[self.label_column_name], self.samples_per_label, None
            )
            train_samples.append(sample_indices)
        # Encode all unique sentences at the indices
        unique_train_indices = list(set(itertools.chain.from_iterable(train_samples)))
        unique_train_dataset = train_split.select(unique_train_indices).select_columns(
            self.input_column_name
        )
        dataloader_train = create_dataloader(
            unique_train_dataset,
            self.metadata,
            input_column=self.input_column_name,
            num_proc=num_proc,
            **encode_kwargs,
        )

        logger.info("Running multilabel classification - Encoding training set...")
        _unique_train_embeddings = model.encode(
            dataloader_train,
            task_metadata=self.metadata,
            hf_split=self.train_split,
            hf_subset=hf_subset,
            **encode_kwargs,
        )
        unique_train_embeddings = dict(
            zip(unique_train_indices, _unique_train_embeddings)
        )
        # Stratified subsampling of test set to 2000 examples.
        test_dataset = eval_split
        try:
            if len(test_dataset) > 2000:
                split_dataset = eval_split.train_test_split(
                    test_size=2000, seed=42, stratify_by_column="label"
                )
                test_dataset = split_dataset["test"]
        except ValueError:
            logger.warning("Couldn't subsample, continuing with the entire test set.")

        dataloader_test = create_dataloader(
            test_dataset.select_columns(self.input_column_name),
            self.metadata,
            input_column=self.input_column_name,
            **encode_kwargs,
        )

        logger.info("Running multilabel classification - Encoding test set...")
        X_test = model.encode(
            dataloader_test,
            task_metadata=self.metadata,
            hf_split=hf_split,
            hf_subset=hf_subset,
            **encode_kwargs,
        )
        binarizer = MultiLabelBinarizer()
        y_test = binarizer.fit_transform(test_dataset[self.label_column_name])

        logger.info("Running multilabel classification - Evaluating classifiers...")
        all_predictions = []
        for _, sample_indices in enumerate(train_samples):
            X_train = np.stack([unique_train_embeddings[idx] for idx in sample_indices])
            y_train = train_split.select(sample_indices)[self.label_column_name]
            y_train = binarizer.transform(y_train)
            y_pred, current_classifier = _evaluate_classifier(
                X_train, y_train, X_test, self.evaluator_model
            )
            if prediction_folder:
                all_predictions.append(y_pred.tolist())

            scores_exp = self._calculate_scores(
                y_test, y_pred, X_test, current_classifier
            )
            scores.append(scores_exp)

        if prediction_folder:
            self._save_task_predictions(
                all_predictions,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        avg_scores: dict[str, Any] = {
            k: np.mean([s[k] for s in scores])  # type: ignore[literal-required]
            for k in scores[0].keys()
        }
        logger.info("Running multilabel classification - Finished.")
        return FullMultilabelClassificationMetrics(
            scores_per_experiment=scores,
            **avg_scores,  # type: ignore[typeddict-item]
        )

    def _calculate_scores(  # type: ignore[override]
        self,
        y_test: NDArray[np.integer],
        y_pred: NDArray[np.integer | np.floating],
        x_test_embedding: Array,
        current_classifier: SklearnModelProtocol,
    ) -> MultilabelClassificationMetrics:
        accuracy = current_classifier.score(x_test_embedding, y_test)
        if isinstance(current_classifier, MultiOutputClassifier):
            predictions = current_classifier.predict_proba(x_test_embedding)
            all_probs = [emb[:, 1] for emb in predictions]

            y_score = np.stack(all_probs, axis=1)  # shape: (n_samples, n_labels)
            lrap = label_ranking_average_precision_score(y_test, y_score)
        else:
            lrap = label_ranking_average_precision_score(y_test, y_pred)
        f1 = f1_score(y_test, y_pred, average="macro")
        hamming = hamming_score(y_test, y_pred)
        return MultilabelClassificationMetrics(
            accuracy=accuracy,
            lrap=lrap,
            f1=f1,
            hamming=hamming,
        )

    def _undersample_data_indices(
        self, y: list[list[int]], samples_per_label: int, idxs: list[int] | None = None
    ) -> tuple[list[int], list[int]]:
        """Undersample data to have samples_per_label samples of each label.

        Currently ensures that each label has at least samples_per_label samples.

        Returns:
            A tuple containing:
                - List of sampled indices.
                - List of all indices after shuffling.
        """
        sample_indices = []
        if idxs is None:
            idxs = list(np.arange(len(y)))
        self.np_rng.shuffle(idxs)
        label_counter: dict[int, int] = defaultdict(int)
        for i in idxs:
            if any((label_counter[label] < samples_per_label) for label in y[i]):
                sample_indices.append(i)
                for label in y[i]:
                    label_counter[label] += 1
        return sample_indices, idxs



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class STS15VisualSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="STS15VisualSTS",
        dataset={
            "path": "Pixel-Linguist/rendered-sts15",
            "revision": "1f8d08d9b9daac7118dfdefeb94b0aac4baf2e5f",
        },
        description="SemEval STS 2015 dataset" + "rendered into images.",
        reference="https://arxiv.org/abs/2402.08183/",
        type="VisualSTS(eng)",
        category="i2i",
        modalities=["image"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cosine_spearman",
        date=("2008-01-01", "2014-07-28"),
        domains=["Blog", "News", "Web", "Written", "Spoken"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="rendered",
        bibtex_citation=r"""
@article{xiao2024pixel,
  author = {Xiao, Chenghao and Huang, Zhuoxu and Chen, Danlu and Hudson, G Thomas and Li, Yizhi and Duan, Haoran and Lin, Chenghua and Fu, Jie and Han, Jungong and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2402.08183},
  title = {Pixel Sentence Representation Learning},
  year = {2024},
}
""",
    )



from .allo_prof_clustering_p2p import AlloProfClusteringP2P, AlloProfClusteringP2PFast
from .allo_prof_clustering_s2s import AlloProfClusteringS2S, AlloProfClusteringS2SFast
from .hal_clustering_s2s import HALClusteringS2S, HALClusteringS2SFast

__all__ = [
    "AlloProfClusteringP2P",
    "AlloProfClusteringP2PFast",
    "AlloProfClusteringS2S",
    "AlloProfClusteringS2SFast",
    "HALClusteringS2S",
    "HALClusteringS2SFast",
]



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FalseFriendsDeEnPC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="FalseFriendsGermanEnglish",
        description="A dataset to identify False Friends / false cognates between English and German. A generally challenging task for multilingual models.",
        reference="https://drive.google.com/file/d/1jgq0nBnV-UiYNxbKNrrr2gxDEHm-DMKH/view?usp=share_link",
        dataset={
            "path": "aari1995/false_friends_de_en_mteb",
            "revision": "15d6c030d3336cbb09de97b2cefc46db93262d40",
        },
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="max_ap",
        date=("2023-08-01", "2023-09-01"),
        domains=["Written"],
        task_subtypes=["False Friends"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@misc{Chibb_2022,
  author = {Chibb, Aaron},
  month = {Sep},
  title = {{German-English False Friends in Multilingual Transformer Models: An Evaluation on Robustness and Word-to-Word Fine-Tuning}},
  year = {2022},
}
""",
    )

    def dataset_transform(self, num_proc: int = 1):
        _dataset = {}
        for split in self.metadata.eval_splits:
            hf_dataset = self.dataset[split]

            _dataset[split] = [
                {
                    "sentence1": hf_dataset["sent1"],
                    "sentence2": hf_dataset["sent2"],
                    "labels": hf_dataset["labels"],
                }
            ]
        self.dataset = _dataset



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class GTSRBClassification(AbsTaskClassification):
    input_column_name = "webp"
    label_column_name = "cls"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="GTSRB",
        description="The German Traffic Sign Recognition Benchmark (GTSRB) is a multi-class classification dataset for traffic signs. It consists of dataset of more than 50,000 traffic sign images. The dataset comprises 43 classes with unbalanced class frequencies.",
        reference="https://benchmark.ini.rub.de/",
        dataset={
            "path": "clip-benchmark/wds_gtsrb",
            "revision": "1c13eff0803d2b02c9dc8dfe85e67770b3f0f3c5",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2011-01-01",
            "2011-12-01",
        ),  # Estimated range for the collection of reviews
        task_subtypes=["Activity recognition"],
        domains=["Scene"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{6033395,
  author = {Stallkamp, Johannes and Schlipsing, Marc and Salmen, Jan and Igel, Christian},
  booktitle = {The 2011 International Joint Conference on Neural Networks},
  doi = {10.1109/IJCNN.2011.6033395},
  keywords = {Humans;Training;Image color analysis;Benchmark testing;Lead;Histograms;Image resolution},
  number = {},
  pages = {1453-1460},
  title = {The German Traffic Sign Recognition Benchmark: A multi-class classification competition},
  volume = {},
  year = {2011},
}
""",
    )



"""Piccolo Chinese embedding models by SenseNova"""

from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

piccolo_base_zh = ModelMeta(
    loader=sentence_transformers_loader,
    name="sensenova/piccolo-base-zh",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=True,
    revision="47c0a63b8f667c3482e05b2fd45577bb19252196",
    release_date="2023-09-04",  # first commit
    n_parameters=None,
    n_embedding_parameters=16_226_304,
    memory_usage_mb=None,  # can't see on model card
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/sensenova/piccolo-base-zh",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,  # They don't specify
)

piccolo_large_zh_v2 = ModelMeta(
    loader=sentence_transformers_loader,
    name="sensenova/piccolo-large-zh-v2",
    model_type=["dense"],
    languages=["zho-Hans"],
    open_weights=False,  # They "temporarily" removed it in may last year
    # "Due to certain internal company considerations"
    revision="05948c1d889355936bdf9db7d30df57dd78d25a3",
    release_date="2024-04-22",  # first commit
    n_parameters=None,
    n_embedding_parameters=None,
    memory_usage_mb=None,  # we don't know because they removed the model
    embed_dim=1024,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/sensenova/piccolo-large-zh-v2",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch"],
    use_instructions=False,
    superseded_by=None,
    adapted_from=None,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,  # They don't say
    citation="""@misc{2405.06932,
    Author = {Junqin Huang and Zhongjie Hu and Zihao Jing and Mengya Gao and Yichao Wu},
    Title = {Piccolo2: General Text Embedding with Multi-task Hybrid Loss Training},
    Year = {2024},
    Eprint = {arXiv:2405.06932},
}""",
)



import csv
from pathlib import Path

from huggingface_hub import snapshot_download

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MLQuestionsRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MLQuestions",
        dataset={
            "path": "McGill-NLP/mlquestions",
            "revision": "83b690cb666c5a8869e7f213a877bbd24a642d7c",
        },
        reference="https://github.com/McGill-NLP/MLQuestions",
        description=(
            "MLQuestions is a domain adaptation dataset for the machine learning domain"
            + "It consists of ML questions along with passages from Wikipedia machine learning pages (https://en.wikipedia.org/wiki/Category:Machine_learning)"
        ),
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["dev", "test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=(
            "2021-01-01",
            "2021-03-31",
        ),  # The period here is for both wiki articles and queries
        domains=["Encyclopaedic", "Academic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-nc-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{kulshreshtha-etal-2021-back,
  address = {Online and Punta Cana, Dominican Republic},
  author = {Kulshreshtha, Devang  and
Belfer, Robert  and
Serban, Iulian Vlad  and
Reddy, Siva},
  booktitle = {Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing},
  month = nov,
  pages = {7064--7078},
  publisher = {Association for Computational Linguistics},
  title = {Back-Training excels Self-Training at Unsupervised Domain Adaptation of Question Generation and Passage Retrieval},
  url = {https://aclanthology.org/2021.emnlp-main.566},
  year = {2021},
}
""",
    )

    def load_data(self, num_proc: int = 1, **kwargs) -> None:
        if self.data_loaded:
            return
        self.corpus, self.queries, self.relevant_docs = {}, {}, {}
        dataset_path = self.metadata.dataset["path"]
        revision = self.metadata.dataset.get("revision", None)
        download_dir = snapshot_download(
            repo_id=dataset_path, repo_type="dataset", revision=revision
        )
        for split in self.metadata.eval_splits:
            corpus, queries, qrels = self._load_data_for_split(download_dir, split)
            self.corpus[split], self.queries[split], self.relevant_docs[split] = (
                corpus,
                queries,
                qrels,
            )

        self.data_loaded = True

    def _load_data_for_split(self, download_dir, split):
        queries, corpus, qrels = {}, {}, {}

        download_dir = Path(download_dir)
        dataset_path = download_dir / f"{split}.csv"
        with dataset_path.open() as csvfile:
            reader = csv.DictReader(csvfile)
            for i, row in enumerate(reader):
                query_id = f"Q{str(i)}"
                doc_id = row["indexes"]
                query = row["target_text"]
                queries[query_id] = query
                qrels[query_id] = {f"C{doc_id}": 1}

        # Same corpus for all splits
        corpus_path = download_dir / "test_passages.csv"
        with corpus_path.open() as csvfile:
            reader = csv.DictReader(csvfile)
            for i, row in enumerate(reader):
                doc_id = f"C{str(i)}"
                corpus[doc_id] = {
                    "title": "",
                    "text": row["input_text"],
                }

        return corpus, queries, qrels



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class MacedonianTweetSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MacedonianTweetSentimentClassification",
        description="An Macedonian dataset for tweet sentiment classification.",
        reference="https://aclanthology.org/R15-1034/",
        dataset={
            "path": "isaacchung/macedonian-tweet-sentiment-classification",
            "revision": "957e075ba35e4417ba7837987fd7053a6533a1a2",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=["2014-11-01", "2015-04-01"],
        eval_splits=["test"],
        eval_langs=["mkd-Cyrl"],
        main_score="accuracy",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{jovanoski-etal-2015-sentiment,
  address = {Hissar, Bulgaria},
  author = {Jovanoski, Dame  and
Pachovski, Veno  and
Nakov, Preslav},
  booktitle = {Proceedings of the International Conference Recent Advances in Natural Language Processing},
  editor = {Mitkov, Ruslan  and
Angelova, Galia  and
Bontcheva, Kalina},
  month = sep,
  pages = {249--257},
  publisher = {INCOMA Ltd. Shoumen, BULGARIA},
  title = {Sentiment Analysis in {T}witter for {M}acedonian},
  url = {https://aclanthology.org/R15-1034},
  year = {2015},
}
""",
        superseded_by="MacedonianTweetSentimentClassification.v2",
    )


class MacedonianTweetSentimentClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MacedonianTweetSentimentClassification.v2",
        description="An Macedonian dataset for tweet sentiment classification. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://aclanthology.org/R15-1034/",
        dataset={
            "path": "mteb/macedonian_tweet_sentiment",
            "revision": "3a8d98dc743809835255f727698d09814b699126",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=["2014-11-01", "2015-04-01"],
        eval_splits=["test"],
        eval_langs=["mkd-Cyrl"],
        main_score="accuracy",
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{jovanoski-etal-2015-sentiment,
  address = {Hissar, Bulgaria},
  author = {Jovanoski, Dame  and
Pachovski, Veno  and
Nakov, Preslav},
  booktitle = {Proceedings of the International Conference Recent Advances in Natural Language Processing},
  editor = {Mitkov, Ruslan  and
Angelova, Galia  and
Bontcheva, Kalina},
  month = sep,
  pages = {249--257},
  publisher = {INCOMA Ltd. Shoumen, BULGARIA},
  title = {Sentiment Analysis in {T}witter for {M}acedonian},
  url = {https://aclanthology.org/R15-1034},
  year = {2015},
}
""",
        adapted_from=["MacedonianTweetSentimentClassification"],
    )



from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta
from mteb.types import PromptType


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    if not instruction or prompt_type == PromptType.document:
        return ""
    if isinstance(instruction, dict):
        if prompt_type is None:
            instruction = next(iter(instruction.values()))  # TODO
        else:
            instruction = instruction[prompt_type]
    return f"Instruct: {instruction}\nQuery:"


multilingual_langs = [
    "afr-Latn",
    "ara-Arab",
    "aze-Latn",
    "bel-Cyrl",
    "bul-Cyrl",
    "ben-Beng",
    "cat-Latn",
    "ceb-Latn",
    "ces-Latn",
    "cym-Latn",
    "dan-Latn",
    "deu-Latn",
    "ell-Grek",
    "eng-Latn",
    "spa-Latn",
    "est-Latn",
    "eus-Latn",
    "fas-Arab",
    "fin-Latn",
    "fra-Latn",
    "glg-Latn",
    "guj-Gujr",
    "heb-Hebr",
    "hin-Deva",
    "hrv-Latn",
    "hat-Latn",
    "hun-Latn",
    "hye-Armn",
    "ind-Latn",
    "isl-Latn",
    "ita-Latn",
    "jpn-Jpan",
    "jav-Latn",
    "kat-Geor",
    "kaz-Cyrl",
    "khm-Khmr",
    "kan-Knda",
    "kor-Hang",
    "kir-Cyrl",
    "lao-Laoo",
    "lit-Latn",
    "lav-Latn",
    "mkd-Cyrl",
    "mal-Mlym",
    "mon-Cyrl",
    "mar-Deva",
    "msa-Latn",
    "mya-Mymr",
    "nep-Deva",
    "nld-Latn",
    "nor-Latn",
    "nob-Latn",
    "nno-Latn",
    "pan-Guru",
    "pol-Latn",
    "por-Latn",
    "que-Latn",
    "ron-Latn",
    "rus-Cyrl",
    "sin-Sinh",
    "slk-Latn",
    "slv-Latn",
    "swa-Latn",
    "tam-Taml",
    "tel-Telu",
    "tha-Thai",
    "tgl-Latn",
    "tur-Latn",
    "ukr-Cyrl",
    "urd-Arab",
    "vie-Latn",
    "yor-Latn",
    "zho-Hans",
]

MOD_CITATION = """@misc{mod-embedding-2025,
  title={MoD-Embedding: A Fine-tuned Multilingual Text Embedding Model},
  author={MoD Team},
  year={2025},
  url={https://huggingface.co/bflhc/MoD-Embedding}
}"""

training_data = {
    "T2Retrieval",
    "DuRetrieval",
    "MMarcoReranking",
    "CMedQAv2-reranking",
    "NQ",
    "MSMARCO",
    "HotpotQA",
    "FEVER",
    "MrTidyRetrieval",
    "MIRACLRetrieval",
    "CodeSearchNet",
}

# Predefined prompts for various RTEB tasks
_PREDEFINED_PROMPTS = {
    # ========== Open Datasets ==========
    # Legal domain
    "AILACasedocs": "Given a legal case scenario, retrieve the most relevant case documents",
    "AILAStatutes": "Given a legal scenario, retrieve the most relevant statute documents",
    "LegalQuAD": "Given a legal question, retrieve relevant legal documents that answer the question",
    "LegalSummarization": "Given a query, retrieve relevant legal documents for summarization",
    # Code domain
    "AppsRetrieval": "Given a query about mobile applications, retrieve relevant app information",
    "HumanEvalRetrieval": "Given a code problem description, retrieve relevant code examples",
    "MBPPRetrieval": "Given a programming problem description, retrieve relevant code solutions",
    "DS1000Retrieval": "Given a data science problem, retrieve relevant code snippets",
    "FreshStackRetrieval": "Given a programming question, retrieve relevant Stack Overflow posts",
    # Finance domain
    "FinQARetrieval": "Given a financial question, retrieve relevant financial documents",
    "FinanceBenchRetrieval": "Given a financial query, retrieve relevant financial information",
    "HC3FinanceRetrieval": "Given a finance-related query, retrieve relevant documents",
    # Medical domain
    "CUREv1": "Given a medical query, retrieve relevant clinical documents",
    "ChatDoctorRetrieval": "Given a medical question, retrieve relevant medical information",
    # SQL domain
    "WikiSQLRetrieval": "Given a natural language query, retrieve relevant SQL examples",
    # Multilingual
    "MIRACLRetrievalHardNegatives": "Given a question, retrieve Wikipedia passages that answer the question",
    # ========== Private/Closed Datasets ==========
    # Code domain (Private)
    "Code1Retrieval": "Given a code problem description, retrieve relevant code examples",
    "JapaneseCode1Retrieval": "Given a code problem description, retrieve relevant code examples",
    # Finance domain (Private)
    "EnglishFinance1Retrieval": "Given a financial query, retrieve relevant financial documents",
    "EnglishFinance2Retrieval": "Given a financial query, retrieve relevant financial documents",
    "EnglishFinance3Retrieval": "Given a financial query, retrieve relevant financial documents",
    "EnglishFinance4Retrieval": "Given a financial query, retrieve relevant financial documents",
    # Healthcare domain (Private)
    "EnglishHealthcare1Retrieval": "Given a medical question, retrieve relevant medical information",
    "GermanHealthcare1Retrieval": "Given a medical question, retrieve relevant medical information",
    # Legal domain (Private)
    "FrenchLegal1Retrieval": "Given a legal query, retrieve relevant legal documents",
    "GermanLegal1Retrieval": "Given a legal query, retrieve relevant legal documents",
    "JapaneseLegal1Retrieval": "Given a legal query, retrieve relevant legal documents",
    # General/Multilingual (Private)
    "French1Retrieval": "Given a query, retrieve relevant passages",
    "German1Retrieval": "Given a query, retrieve relevant passages",
}


MoD_Embedding = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        apply_instruction_to_passages=False,
        prompts_dict=_PREDEFINED_PROMPTS,
        max_seq_length=18480,
        model_kwargs={"torch_dtype": "bfloat16"},
    ),
    name="bflhc/MoD-Embedding",
    languages=multilingual_langs,
    open_weights=True,
    revision="acbb5b70fdab262226a6af2bc62001de8021b05c",
    release_date="2025-12-14",
    n_parameters=4021774336,
    n_embedding_parameters=None,
    memory_usage_mb=7671,
    embed_dim=2560,
    max_tokens=32768,
    license="apache-2.0",
    reference="https://huggingface.co/bflhc/MoD-Embedding",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=training_data,
    citation=MOD_CITATION,
    adapted_from="Qwen/Qwen3-Embedding-4B",
)



import itertools

from datasets import Dataset, DatasetDict

from mteb.abstasks.clustering import (
    AbsTaskClustering,
    _check_label_distribution,
)
from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata


class MedrxivClusteringS2SFast(AbsTaskClustering):
    metadata = TaskMetadata(
        name="MedrxivClusteringS2S.v2",
        description="Clustering of titles from medrxiv across 51 categories.",
        reference="https://api.medrxiv.org/",
        dataset={
            "path": "mteb/medrxiv-clustering-s2s",
            "revision": "35191c8c0dca72d8ff3efcd72aa802307d469663",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Medical", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.medrxiv.org/content/about-medrxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        prompt="Identify the main category of Medrxiv papers based on the titles",
        adapted_from=["MedrxivClusteringS2S"],
    )

    def dataset_transform(self, num_proc: int = 1):
        ds = {}
        for split in self.metadata.eval_splits:
            labels = list(itertools.chain.from_iterable(self.dataset[split]["labels"]))
            sentences = list(
                itertools.chain.from_iterable(self.dataset[split]["sentences"])
            )
            _check_label_distribution(self.dataset[split])
            ds[split] = Dataset.from_dict({"labels": labels, "sentences": sentences})
        self.dataset = DatasetDict(ds)


class MedrxivClusteringS2S(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="MedrxivClusteringS2S",
        description="Clustering of titles from medrxiv. Clustering of 10 sets, based on the main category.",
        reference="https://api.medrxiv.org/",
        dataset={
            "path": "mteb/medrxiv-clustering-s2s",
            "revision": "35191c8c0dca72d8ff3efcd72aa802307d469663",
        },
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2021-01-01", "2022-05-10"),
        domains=["Academic", "Medical", "Written"],
        task_subtypes=["Thematic clustering"],
        license="https://www.medrxiv.org/content/about-medrxiv",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation="",
        prompt="Identify the main category of Medrxiv papers based on the titles",
        superseded_by="MedrxivClusteringS2S.v2",
    )



"""Farsi/Persian models for evaluation on the Persian part of MTEB"""

from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

parsbert = ModelMeta(
    loader=sentence_transformers_loader,
    name="HooshvareLab/bert-base-parsbert-uncased",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="d73a0e2c7492c33bd5819bcdb23eba207404dd19",
    release_date="2021-05-19",
    n_parameters=162_841_344,
    n_embedding_parameters=76_800_000,
    memory_usage_mb=621,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/HooshvareLab/bert-base-parsbert-uncased",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # It's just a base model
        # https://github.com/miras-tech/MirasText/tree/master/MirasText
        # Persian Wikipedia
        # Other data crawled from websites like bigbangpage.com, chetor.com, eligasht.com/blog, digikala.com/mag, and ted.com/talks.
    ),
    citation="""
    @article{ParsBERT,
    title={ParsBERT: Transformer-based Model for Persian Language Understanding},
    author={Mehrdad Farahani, Mohammad Gharachorloo, Marzieh Farahani, Mohammad Manthouri},
    journal={ArXiv},
    year={2020},
    volume={abs/2005.12515}
}
""",
)

bert_zwnj = ModelMeta(
    loader=sentence_transformers_loader,
    name="m3hrdadfi/bert-zwnj-wnli-mean-tokens",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="b9506ddc579ac8c398ae6dae680401ae0a1a5b23",
    release_date="2021-06-28",
    n_parameters=118_297_344,
    n_embedding_parameters=32_256_000,
    memory_usage_mb=451,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/m3hrdadfi/bert-zwnj-wnli-mean-tokens",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        # This model is finetuned from HooshvareLab/bert-base-parsbert-uncased
        "FarsTail",
        # https://github.com/m3hrdadfi/sentence-transformers?tab=readme-ov-file
    },
)

roberta_zwnj = ModelMeta(
    loader=sentence_transformers_loader,
    name="m3hrdadfi/roberta-zwnj-wnli-mean-tokens",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="36f912ac44e22250aee16ea533a4ff8cd848c1a1",
    release_date="2021-06-28",
    n_parameters=118_298_112,
    n_embedding_parameters=32_256_000,
    memory_usage_mb=451,
    embed_dim=768,
    license="not specified",
    max_tokens=514,
    reference="https://huggingface.co/m3hrdadfi/roberta-zwnj-wnli-mean-tokens",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets={
        "FarsTail",
        # https://github.com/m3hrdadfi/sentence-transformers?tab=readme-ov-file
    },
)

sentence_transformer_parsbert = ModelMeta(
    loader=sentence_transformers_loader,
    name="myrkur/sentence-transformer-parsbert-fa",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="72bd0a3557622f0ae08a092f4643609e0b950cdd",
    release_date="2024-12-10",
    n_parameters=162_841_344,
    n_embedding_parameters=76_800_000,
    memory_usage_mb=621,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/myrkur/sentence-transformer-parsbert-fa",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # This model is finetuned from HooshvareLab/bert-base-parsbert-uncased
        # https://huggingface.co/datasets/Gholamreza/pquad
    ),
)

tooka_bert_base = ModelMeta(
    loader=sentence_transformers_loader,
    name="PartAI/TookaBERT-Base",
    languages=["fas-Arab"],
    open_weights=True,
    revision="fa5ca89df5670700d9325b8872ac65c17cb24582",
    release_date="2024-12-08",
    n_parameters=122_905_344,
    n_embedding_parameters=36_864_000,
    memory_usage_mb=469,
    embed_dim=768,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/PartAI/TookaBERT-Base",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # It's just a base model
        # https://huggingface.co/datasets/sbunlp/hmblogs-v3
        # https://huggingface.co/datasets/Targoman/TLPC
        # https://huggingface.co/datasets/allenai/MADLAD-400 (cleaned Persian subset)
    ),
)

tooka_sbert = ModelMeta(
    loader=sentence_transformers_loader,
    name="PartAI/Tooka-SBERT",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="5d07f0c543aca654373b931ae07cd197769110fd",
    release_date="2024-12-07",
    n_parameters=353_039_360,
    n_embedding_parameters=49_152_000,
    memory_usage_mb=1347,
    embed_dim=1024,
    license="apache-2.0",
    max_tokens=512,
    reference="https://huggingface.co/PartAI/Tooka-SBERT",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)

fa_bert = ModelMeta(
    loader=sentence_transformers_loader,
    name="sbunlp/fabert",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="a0e3973064c97768e121b9b95f21adc94e0ca3fb",
    release_date="2024-10-07",
    n_parameters=124_441_344,
    n_embedding_parameters=38_400_000,
    memory_usage_mb=475,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/sbunlp/fabert",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "Transformers", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=set(
        # It's just a base model
        # https://huggingface.co/datasets/sbunlp/hmblogs-v3
    ),
    citation="""@inproceedings{masumi-etal-2025-fabert,
    title = "{F}a{BERT}: Pre-training {BERT} on {P}ersian Blogs",
    author = "Masumi, Mostafa  and
      Majd, Seyed Soroush  and
      Shamsfard, Mehrnoush  and
      Beigy, Hamid",
    editor = "Bak, JinYeong  and
      Goot, Rob van der  and
      Jang, Hyeju  and
      Buaphet, Weerayut  and
      Ramponi, Alan  and
      Xu, Wei  and
      Ritter, Alan",
    booktitle = "Proceedings of the Tenth Workshop on Noisy and User-generated Text",
    month = may,
    year = "2025",
    address = "Albuquerque, New Mexico, USA",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2025.wnut-1.10/",
    doi = "10.18653/v1/2025.wnut-1.10",
    pages = "85--96",
    ISBN = "979-8-89176-232-9",
}""",
)

tooka_sbert_v2_small = ModelMeta(
    loader=sentence_transformers_loader,
    name="PartAI/Tooka-SBERT-V2-Small",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="8bbed87e36669387f71437c061430ba56d1b496f",
    release_date="2025-05-01",
    n_parameters=122_905_344,
    n_embedding_parameters=36_864_000,
    memory_usage_mb=496,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/PartAI/Tooka-SBERT-V2-Small",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)

tooka_sbert_v2_large = ModelMeta(
    loader=sentence_transformers_loader,
    name="PartAI/Tooka-SBERT-V2-Large",
    model_type=["dense"],
    languages=["fas-Arab"],
    open_weights=True,
    revision="b59682efa961122cc0e4408296d5852870c82eae",
    release_date="2025-05-01",
    n_parameters=353_039_360,
    n_embedding_parameters=49_152_000,
    memory_usage_mb=1347,
    embed_dim=1024,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/PartAI/Tooka-SBERT-V2-Large",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    citation="""@inproceedings{reimers-2019-sentence-bert,
    title = "Sentence-BERT: Sentence Embeddings using Siamese BERT-Networks",
    author = "Reimers, Nils and Gurevych, Iryna",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing",
    month = "11",
    year = "2019",
    publisher = "Association for Computational Linguistics",
    url = "https://arxiv.org/abs/1908.10084",
}""",
)



from .ask_ubuntu_dup_questions import AskUbuntuDupQuestions
from .built_bench_reranking import BuiltBenchReranking
from .hume_core17_instruction_reranking import HUMECore17InstructionReranking
from .hume_news21_instruction_reranking import HUMENews21InstructionReranking
from .hume_robust04_instruction_reranking import HUMERobust04InstructionReranking
from .loc_bench_reranking import LocBenchReranking
from .mind_small_reranking import MindSmallReranking
from .multi_swe_bench_reranking import MultiSWEbenchReranking
from .nev_ir import NevIR
from .sci_docs_reranking import SciDocsReranking
from .stack_overflow_dup_questions import StackOverflowDupQuestions
from .swe_bench_lite_reranking import SWEbenchLiteReranking
from .swe_bench_multilingual_reranking import SWEbenchMultilingualRR
from .swe_bench_verified_reranking import SWEbenchVerifiedReranking
from .swe_poly_bench_reranking import SWEPolyBenchReranking
from .web_linx_candidates_reranking import WebLINXCandidatesReranking

__all__ = [
    "AskUbuntuDupQuestions",
    "BuiltBenchReranking",
    "HUMECore17InstructionReranking",
    "HUMENews21InstructionReranking",
    "HUMERobust04InstructionReranking",
    "LocBenchReranking",
    "MindSmallReranking",
    "MultiSWEbenchReranking",
    "NevIR",
    "SWEPolyBenchReranking",
    "SWEbenchLiteReranking",
    "SWEbenchMultilingualRR",
    "SWEbenchVerifiedReranking",
    "SciDocsReranking",
    "StackOverflowDupQuestions",
    "WebLINXCandidatesReranking",
]



from __future__ import annotations

from mteb.models import ModelMeta, sentence_transformers_loader
from mteb.types import PromptType

english_code_retriever = ModelMeta(
    loader=sentence_transformers_loader,
    loader_kwargs=dict(
        model_prompts={
            PromptType.query.value: "search_query: ",
            PromptType.document.value: "search_document: ",
        },
    ),
    name="fyaronskiy/english_code_retriever",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="be653fab7d27a7348a0c2c3d16b9f92a7f10cb0c",
    release_date="2025-07-10",
    n_parameters=149_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=568,
    embed_dim=768,
    license="mit",
    max_tokens=8192,
    reference="https://huggingface.co/fyaronskiy/english_code_retriever",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/code-search-net/code_search_net",
    training_datasets={"CodeSearchNet"},
)



"""ATLES Champion Embedding Model for MTEB."""

from mteb.models.model_meta import ModelMeta
from mteb.models.sentence_transformer_wrapper import sentence_transformers_loader

spartan8806_atles_champion_embedding = ModelMeta(
    loader=sentence_transformers_loader,
    name="spartan8806/atles-champion-embedding",
    model_type=["dense"],
    languages=["eng-Latn"],
    open_weights=True,
    revision="d4c74d7000bbd25f3597fc0f2dcde59ef1386e8f",
    release_date="2025-11-15",
    n_parameters=110_000_000,
    n_embedding_parameters=23_444_736,
    memory_usage_mb=420,
    max_tokens=512,
    embed_dim=768,
    license="apache-2.0",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "safetensors", "Transformers"],
    reference="https://huggingface.co/spartan8806/atles-champion-embedding",
    use_instructions=False,
    training_datasets={"STSBenchmark"},
    adapted_from="sentence-transformers/all-mpnet-base-v2",
    public_training_code=None,
    public_training_data=None,
    citation="""@article{conner2025epistemic,
  title={The Epistemic Barrier: How RLHF Makes AI Consciousness Empirically Undecidable},
  author={Conner (spartan8806)},
  journal={ATLES Research Papers},
  year={2025},
  note={Cross-model validation study (Phoenix, Grok, Gemini, Claude)}
}""",
)



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_image_dependencies, requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

LLM2CLIP_CITATION = """@misc{huang2024llm2clippowerfullanguagemodel,
  title={LLM2CLIP: Powerful Language Model Unlock Richer Visual Representation},
  author={Weiquan Huang and Aoqi Wu and Yifan Yang and Xufang Luo and Yuqing Yang and Liang Hu and Qi Dai and Xiyang Dai and Dongdong Chen and Chong Luo and Lili Qiu},
  year={2024},
  eprint={2411.04997},
  archivePrefix={arXiv},
  primaryClass={cs.CV},
  url={https://arxiv.org/abs/2411.04997}
}"""

MODEL2PROCESSOR = {
    "microsoft/LLM2CLIP-Openai-L-14-336": "openai/clip-vit-large-patch14-336",
    "microsoft/LLM2CLIP-Openai-B-16": "openai/clip-vit-base-patch16",
    "microsoft/LLM2CLIP-Openai-L-14-224": "openai/clip-vit-large-patch14",
}


def llm2clip_loader(model_name, **kwargs):
    from transformers import AutoConfig, AutoModel, AutoTokenizer, CLIPImageProcessor

    requires_package(
        llm2clip_loader, "llm2vec", model_name, "pip install 'mteb[llm2vec]'"
    )
    from llm2vec import LLM2Vec

    class LLM2CLIPAbsEncoder(AbsEncoder):
        def __init__(
            self,
            model_name: str,
            device: str = "cuda" if torch.cuda.is_available() else "cpu",
            **kwargs: Any,
        ):
            requires_image_dependencies()

            if model_name not in MODEL2PROCESSOR:
                raise Exception(
                    f"This model {model_name} is not in the supported mode list: {list(MODEL2PROCESSOR.keys())}."
                )

            self.device = device
            from huggingface_hub import snapshot_download

            model_folder_path = snapshot_download(
                repo_id=model_name, allow_patterns=["*.json", "*.safetensors", "*.py"]
            )
            snapshot_download(
                repo_id=MODEL2PROCESSOR[model_name],
                allow_patterns=["*.json", "*.safetensors", "*.py"],
            )
            model_name_or_path = Path(model_folder_path)
            self.processor = CLIPImageProcessor.from_pretrained(
                MODEL2PROCESSOR[model_name]
            )
            self.model = (
                AutoModel.from_pretrained(
                    model_name_or_path,
                    torch_dtype=torch.float16,
                    trust_remote_code=True,
                )
                .to(self.device)
                .eval()
            )

            llm_model_name = (
                "microsoft/LLM2CLIP-Llama-3-8B-Instruct-CC-Finetuned"  # constant
            )
            config = AutoConfig.from_pretrained(llm_model_name, trust_remote_code=True)
            llm_model = AutoModel.from_pretrained(
                llm_model_name,
                torch_dtype=torch.bfloat16,
                config=config,
                trust_remote_code=True,
            )
            tokenizer = AutoTokenizer.from_pretrained(llm_model_name)
            llm_model.config._name_or_path = "meta-llama/Meta-Llama-3-8B-Instruct"  # Workaround for LLM2VEC. constant.
            self.l2v = LLM2Vec(
                llm_model,
                tokenizer,
                pooling_mode="mean",
                max_length=512,
                doc_max_length=512,
            )

        def get_text_embeddings(
            self,
            texts: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_text_embeddings = []

            with torch.no_grad(), torch.amp.autocast("cuda"):
                for batch in tqdm(
                    texts, disable=not show_progress_bar, desc="Text Encoding"
                ):
                    text_features = self.l2v.encode(
                        batch["text"], convert_to_tensor=True
                    ).to(self.device)
                    text_features = self.model.get_text_features(text_features)
                    text_features /= text_features.norm(dim=-1, keepdim=True)
                    all_text_embeddings.append(text_features.cpu().to(torch.float32))

            all_text_embeddings = torch.cat(all_text_embeddings, dim=0)

            return all_text_embeddings

        def get_image_embeddings(
            self,
            images: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            **kwargs: Any,
        ):
            all_image_embeddings = []

            with torch.no_grad(), torch.amp.autocast("cuda"):
                for batch in tqdm(
                    images, disable=not show_progress_bar, desc="Image Encoding"
                ):
                    input_pixels = self.processor(
                        images=batch["image"],
                        return_tensors="pt",
                    ).pixel_values.to(self.device)
                    image_features = self.model.get_image_features(input_pixels)
                    image_features /= image_features.norm(dim=-1, keepdim=True)
                    all_image_embeddings.append(image_features.cpu().to(torch.float32))

            all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
            return all_image_embeddings

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            **kwargs: Any,
        ) -> Array:
            text_embeddings = None
            image_embeddings = None
            if "text" in inputs.dataset.features:
                text_embeddings = self.get_text_embeddings(inputs, **kwargs)
            if "image" in inputs.dataset.features:
                image_embeddings = self.get_image_embeddings(inputs, **kwargs)

            if text_embeddings is not None and image_embeddings is not None:
                if len(text_embeddings) != len(image_embeddings):
                    raise ValueError(
                        "The number of texts and images must have the same length"
                    )
                fused_embeddings = text_embeddings + image_embeddings
                return fused_embeddings
            elif text_embeddings is not None:
                return text_embeddings
            elif image_embeddings is not None:
                return image_embeddings
            raise ValueError

    return LLM2CLIPAbsEncoder(model_name, **kwargs)


llm2clip_training_sets = set(
    # CC3M
    # CC12M
    # YFCC15M
    # Recap-DataComp-1B(30M subset)
)

llm2clip_openai_l_14_336 = ModelMeta(
    loader=llm2clip_loader,
    name="microsoft/LLM2CLIP-Openai-L-14-336",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="92512331f393a003c3d98404677f991c188162c9",
    release_date="2024-11-07",
    modalities=["image", "text"],
    n_parameters=579_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=1280,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/microsoft/LLM2CLIP",
    public_training_data=None,
    framework=["PyTorch", "safetensors"],
    reference="https://huggingface.co/microsoft/LLM2CLIP-Openai-L-14-336",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=llm2clip_training_sets,
    citation=LLM2CLIP_CITATION,
)

# NOTE: https://huggingface.co/microsoft/LLM2CLIP-Openai-L-14-224/discussions/1
llm2clip_openai_l_14_224 = ModelMeta(
    loader=llm2clip_loader,
    name="microsoft/LLM2CLIP-Openai-L-14-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="6b8a11a94ff380fa220dfefe73ac9293d2677575",
    release_date="2024-11-07",
    modalities=["image", "text"],
    n_parameters=578_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=1280,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/microsoft/LLM2CLIP",
    public_training_data=None,
    framework=["PyTorch", "safetensors"],
    reference="https://huggingface.co/microsoft/LLM2CLIP-Openai-L-14-224",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=llm2clip_training_sets,
    citation=LLM2CLIP_CITATION,
)

llm2clip_openai_b_16 = ModelMeta(
    loader=llm2clip_loader,
    name="microsoft/LLM2CLIP-Openai-B-16",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="ecfb347eb3dcfeb2fbc2a2eae7de6ac5a001aaf8",
    release_date="2024-11-07",
    modalities=["image", "text"],
    n_parameters=361_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=None,
    embed_dim=1280,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/microsoft/LLM2CLIP",
    public_training_data=None,
    framework=["PyTorch", "safetensors"],
    reference="https://huggingface.co/microsoft/LLM2CLIP-Openai-B-16",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=True,
    training_datasets=llm2clip_training_sets,
    citation=LLM2CLIP_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class LocBenchReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LocBenchRR",
        description="Software Issue Localization.",
        reference="https://arxiv.org/abs/2503.09089",
        dataset={
            "path": "mteb/LocBenchRR",
            "revision": "4a1fe7be94481b1e5a47d072782c89b146b4fecf",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn", "python-Code"],
        main_score="recall_at_10",
        date=("2025-03-12", "2025-03-12"),  # arxiv v1 submission date
        domains=["Programming", "Written"],
        task_subtypes=["Code retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        prompt={
            "query": "Given a github issue, identify the code that needs to be changed to fix the issue."
        },
        bibtex_citation=r"""
@misc{chen2025locagentgraphguidedllmagents,
  archiveprefix = {arXiv},
  author = {Zhaoling Chen and Xiangru Tang and Gangda Deng and Fang Wu and Jialong Wu and Zhiwei Jiang and Viktor Prasanna and Arman Cohan and Xingyao Wang},
  eprint = {2503.09089},
  primaryclass = {cs.SE},
  title = {LocAgent: Graph-Guided LLM Agents for Code Localization},
  url = {https://arxiv.org/abs/2503.09089},
  year = {2025},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class BengaliHateSpeechClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="BengaliHateSpeechClassification",
        description="The Bengali Hate Speech Dataset is a Bengali-language dataset of news articles collected from various Bengali media sources and categorized based on the type of hate in the text.",
        reference="https://huggingface.co/datasets/bn_hate_speech",
        dataset={
            "path": "rezacsedu/bn_hate_speech",
            "revision": "99612296bc093f0720cac7d7cbfcb67eecf1ca2f",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["ben-Beng"],
        main_score="f1",
        date=("2019-12-01", "2020-04-09"),
        dialect=[],
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="expert-annotated",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{karim2020BengaliNLP,
  author = {Karim, Md. Rezaul and Chakravarti, Bharathi Raja and P. McCrae, John and Cochez, Michael},
  booktitle = {7th IEEE International Conference on Data Science and Advanced Analytics (IEEE DSAA,2020)},
  publisher = {IEEE},
  title = {Classification Benchmarks for Under-resourced Bengali Language based on Multichannel Convolutional-LSTM Network},
  year = {2020},
}
""",
        superseded_by="BengaliHateSpeechClassification.v2",
    )

    def dataset_transform(self, num_proc: int = 1):
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["train"]
        )


class BengaliHateSpeechClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="BengaliHateSpeechClassification.v2",
        description="The Bengali Hate Speech Dataset is a Bengali-language dataset of news articles collected from various Bengali media sources and categorized based on the type of hate in the text. This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2632)",
        reference="https://huggingface.co/datasets/bn_hate_speech",
        dataset={
            "path": "mteb/bengali_hate_speech",
            "revision": "ccd942d16d2f0512799a1480034650a7b076027e",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["ben-Beng"],
        main_score="f1",
        date=("2019-12-01", "2020-04-09"),
        dialect=[],
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="mit",
        annotations_creators="expert-annotated",
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{karim2020BengaliNLP,
  author = {Karim, Md. Rezaul and Chakravarti, Bharathi Raja and P. McCrae, John and Cochez, Michael},
  booktitle = {7th IEEE International Conference on Data Science and Advanced Analytics (IEEE DSAA,2020)},
  publisher = {IEEE},
  title = {Classification Benchmarks for Under-resourced Bengali Language based on Multichannel Convolutional-LSTM Network},
  year = {2020},
}
""",
    )



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FORBI2I(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="FORBI2IRetrieval",
        description="Retrieve flat object images from 8 classes.",
        reference="https://github.com/pxiangwu/FORB",
        dataset={
            "path": "isaacchung/forb_retrieval",
            "revision": "26ab4bd972854becada339afc80f5f3ffc047e2b",
        },
        type="Any2AnyRetrieval",
        category="i2i",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cv_recall_at_1",
        date=("2022-01-01", "2023-01-01"),
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{wu2023forbflatobjectretrieval,
  archiveprefix = {arXiv},
  author = {Pengxiang Wu and Siman Wang and Kevin Dela Rosa and Derek Hao Hu},
  eprint = {2309.16249},
  primaryclass = {cs.CV},
  title = {FORB: A Flat Object Retrieval Benchmark for Universal Image Embedding},
  url = {https://arxiv.org/abs/2309.16249},
  year = {2023},
}
""",
    )



from .amazon_polarity_classification import (
    AmazonPolarityClassification,
    AmazonPolarityClassificationV2,
)
from .arxiv_classification import ArxivClassification, ArxivClassificationV2
from .banking77_classification import Banking77Classification, Banking77ClassificationV2
from .birdsnap_classification import BirdsnapClassification
from .caltech101_classification import Caltech101Classification
from .cifar import CIFAR10Classification, CIFAR100Classification
from .country211_classification import Country211Classification
from .dbpedia_classification import DBpediaClassification, DBpediaClassificationV2
from .dtd_classification import DTDClassification
from .emotion_classification import EmotionClassification, EmotionClassificationV2
from .euro_sat_classification import EuroSATClassification
from .fer2013_classification import FER2013Classification
from .fgvc_aircraft_classification import FGVCAircraftClassification
from .financial_phrasebank_classification import (
    FinancialPhrasebankClassification,
    FinancialPhrasebankClassificationV2,
)
from .food101_classification import Food101Classification
from .frenk_en_classification import FrenkEnClassification, FrenkEnClassificationV2
from .gtsrb_classification import GTSRBClassification
from .hume_emotion_classification import HUMEEmotionClassification
from .hume_toxic_conversations_classification import (
    HUMEToxicConversationsClassification,
)
from .hume_tweet_sentiment_extraction_classification import (
    HUMETweetSentimentExtractionClassification,
)
from .imagenet1k import Imagenet1kClassification
from .imdb_classification import ImdbClassification, ImdbClassificationV2
from .legal_bench_classification import (
    CanadaTaxCourtOutcomesLegalBenchClassification,
    ContractNLIConfidentialityOfAgreementLegalBenchClassification,
    ContractNLIExplicitIdentificationLegalBenchClassification,
    ContractNLIInclusionOfVerballyConveyedInformationLegalBenchClassification,
    ContractNLILimitedUseLegalBenchClassification,
    ContractNLINoLicensingLegalBenchClassification,
    ContractNLINoticeOnCompelledDisclosureLegalBenchClassification,
    ContractNLIPermissibleAcquirementOfSimilarInformationLegalBenchClassification,
    ContractNLIPermissibleCopyLegalBenchClassification,
    ContractNLIPermissibleDevelopmentOfSimilarInformationLegalBenchClassification,
    ContractNLIPermissiblePostAgreementPossessionLegalBenchClassification,
    ContractNLIReturnOfConfidentialInformationLegalBenchClassification,
    ContractNLISharingWithEmployeesLegalBenchClassification,
    ContractNLISharingWithThirdPartiesLegalBenchClassification,
    ContractNLISurvivalOfObligationsLegalBenchClassification,
    CorporateLobbyingLegalBenchClassification,
    CUADAffiliateLicenseLicenseeLegalBenchClassification,
    CUADAffiliateLicenseLicensorLegalBenchClassification,
    CUADAntiAssignmentLegalBenchClassification,
    CUADAuditRightsLegalBenchClassification,
    CUADCapOnLiabilityLegalBenchClassification,
    CUADChangeOfControlLegalBenchClassification,
    CUADCompetitiveRestrictionExceptionLegalBenchClassification,
    CUADCovenantNotToSueLegalBenchClassification,
    CUADEffectiveDateLegalBenchClassification,
    CUADExclusivityLegalBenchClassification,
    CUADExpirationDateLegalBenchClassification,
    CUADGoverningLawLegalBenchClassification,
    CUADInsuranceLegalBenchClassification,
    CUADIPOwnershipAssignmentLegalBenchClassification,
    CUADIrrevocableOrPerpetualLicenseLegalBenchClassification,
    CUADJointIPOwnershipLegalBenchClassification,
    CUADLicenseGrantLegalBenchClassification,
    CUADLiquidatedDamagesLegalBenchClassification,
    CUADMinimumCommitmentLegalBenchClassification,
    CUADMostFavoredNationLegalBenchClassification,
    CUADNonCompeteLegalBenchClassification,
    CUADNonDisparagementLegalBenchClassification,
    CUADNonTransferableLicenseLegalBenchClassification,
    CUADNoSolicitOfCustomersLegalBenchClassification,
    CUADNoSolicitOfEmployeesLegalBenchClassification,
    CUADNoticePeriodToTerminateRenewalLegalBenchClassification,
    CUADPostTerminationServicesLegalBenchClassification,
    CUADPriceRestrictionsLegalBenchClassification,
    CUADRenewalTermLegalBenchClassification,
    CUADRevenueProfitSharingLegalBenchClassification,
    CUADRofrRofoRofnLegalBenchClassification,
    CUADSourceCodeEscrowLegalBenchClassification,
    CUADTerminationForConvenienceLegalBenchClassification,
    CUADThirdPartyBeneficiaryLegalBenchClassification,
    CUADUncappedLiabilityLegalBenchClassification,
    CUADUnlimitedAllYouCanEatLicenseLegalBenchClassification,
    CUADVolumeRestrictionLegalBenchClassification,
    CUADWarrantyDurationLegalBenchClassification,
    DefinitionClassificationLegalBenchClassification,
    Diversity1LegalBenchClassification,
    Diversity2LegalBenchClassification,
    Diversity3LegalBenchClassification,
    Diversity4LegalBenchClassification,
    Diversity5LegalBenchClassification,
    Diversity6LegalBenchClassification,
    FunctionOfDecisionSectionLegalBenchClassification,
    InsurancePolicyInterpretationLegalBenchClassification,
    InternationalCitizenshipQuestionsLegalBenchClassification,
    JCrewBlockerLegalBenchClassification,
    JCrewBlockerLegalBenchClassificationV2,
    LearnedHandsBenefitsLegalBenchClassification,
    LearnedHandsBusinessLegalBenchClassification,
    LearnedHandsConsumerLegalBenchClassification,
    LearnedHandsCourtsLegalBenchClassification,
    LearnedHandsCrimeLegalBenchClassification,
    LearnedHandsDivorceLegalBenchClassification,
    LearnedHandsDomesticViolenceLegalBenchClassification,
    LearnedHandsEducationLegalBenchClassification,
    LearnedHandsEmploymentLegalBenchClassification,
    LearnedHandsEstatesLegalBenchClassification,
    LearnedHandsFamilyLegalBenchClassification,
    LearnedHandsHealthLegalBenchClassification,
    LearnedHandsHousingLegalBenchClassification,
    LearnedHandsImmigrationLegalBenchClassification,
    LearnedHandsTortsLegalBenchClassification,
    LearnedHandsTrafficLegalBenchClassification,
    LegalReasoningCausalityLegalBenchClassification,
    LegalReasoningCausalityLegalBenchClassificationV2,
    MAUDLegalBenchClassification,
    MAUDLegalBenchClassificationV2,
    NYSJudicialEthicsLegalBenchClassification,
    OPP115DataRetentionLegalBenchClassification,
    OPP115DataSecurityLegalBenchClassification,
    OPP115DataSecurityLegalBenchClassificationV2,
    OPP115DoNotTrackLegalBenchClassification,
    OPP115DoNotTrackLegalBenchClassificationV2,
    OPP115FirstPartyCollectionUseLegalBenchClassification,
    OPP115InternationalAndSpecificAudiencesLegalBenchClassification,
    OPP115PolicyChangeLegalBenchClassification,
    OPP115ThirdPartySharingCollectionLegalBenchClassification,
    OPP115UserAccessEditAndDeletionLegalBenchClassification,
    OPP115UserChoiceControlLegalBenchClassification,
    OPP115UserChoiceControlLegalBenchClassificationV2,
    OralArgumentQuestionPurposeLegalBenchClassification,
    OralArgumentQuestionPurposeLegalBenchClassificationV2,
    OverrulingLegalBenchClassification,
    OverrulingLegalBenchClassificationV2,
    PersonalJurisdictionLegalBenchClassification,
    PROALegalBenchClassification,
    SCDBPAccountabilityLegalBenchClassification,
    SCDBPAuditsLegalBenchClassification,
    SCDBPCertificationLegalBenchClassification,
    SCDBPTrainingLegalBenchClassification,
    SCDBPVerificationLegalBenchClassification,
    SCDDAccountabilityLegalBenchClassification,
    SCDDAuditsLegalBenchClassification,
    SCDDCertificationLegalBenchClassification,
    SCDDTrainingLegalBenchClassification,
    SCDDVerificationLegalBenchClassification,
    TelemarketingSalesRuleLegalBenchClassification,
    TextualismToolDictionariesLegalBenchClassification,
    TextualismToolPlainLegalBenchClassification,
    UCCVCommonLawLegalBenchClassification,
    UnfairTOSLegalBenchClassification,
)
from .mnist_classification import MNISTClassification
from .news_classification import NewsClassification, NewsClassificationV2
from .oxford_flowers_classification import OxfordFlowersClassification
from .oxford_pets_classification import OxfordPetsClassification
from .patch_camelyon_classification import PatchCamelyonClassification
from .patent_classification import PatentClassification, PatentClassificationV2
from .poem_sentiment_classification import (
    PoemSentimentClassification,
    PoemSentimentClassificationV2,
)
from .resisc45_classification import RESISC45Classification
from .sds_eye_protection_classification import (
    SDSEyeProtectionClassification,
    SDSEyeProtectionClassificationV2,
)
from .sds_gloves_classification import (
    SDSGlovesClassification,
    SDSGlovesClassificationV2,
)
from .stanford_cars_classification import StanfordCarsClassification
from .stl10_classification import STL10Classification
from .sun397_classification import SUN397Classification
from .toxic_chat_classification import (
    ToxicChatClassification,
    ToxicChatClassificationV2,
)
from .toxic_conversations_classification import (
    ToxicConversationsClassification,
    ToxicConversationsClassificationV2,
)
from .tweet_sentiment_extraction_classification import (
    TweetSentimentExtractionClassification,
    TweetSentimentExtractionClassificationV2,
)
from .tweet_topic_single_classification import (
    TweetTopicSingleClassification,
    TweetTopicSingleClassificationV2,
)
from .ucf101_classification import UCF101Classification
from .wikipedia_bio_met_chem_classification import (
    WikipediaBioMetChemClassification,
    WikipediaBioMetChemClassificationV2,
)
from .wikipedia_biolum_neurochem_classification import (
    WikipediaBiolumNeurochemClassification,
)
from .wikipedia_chem_eng_specialties_classification import (
    WikipediaChemEngSpecialtiesClassification,
)
from .wikipedia_chem_fields_classification import (
    WikipediaChemFieldsClassification,
    WikipediaChemFieldsClassificationV2,
)
from .wikipedia_chemistry_topics_classification import (
    WikipediaChemistryTopicsClassification,
)
from .wikipedia_comp_chem_spectroscopy_classification import (
    WikipediaCompChemSpectroscopyClassification,
    WikipediaCompChemSpectroscopyClassificationV2,
)
from .wikipedia_cryobiology_separation_classification import (
    WikipediaCryobiologySeparationClassification,
)
from .wikipedia_crystallography_analytical_classification import (
    WikipediaCrystallographyAnalyticalClassification,
    WikipediaCrystallographyAnalyticalClassificationV2,
)
from .wikipedia_greenhouse_enantiopure_classification import (
    WikipediaGreenhouseEnantiopureClassification,
)
from .wikipedia_isotopes_fission_classification import (
    WikipediaIsotopesFissionClassification,
)
from .wikipedia_luminescence_classification import WikipediaLuminescenceClassification
from .wikipedia_organic_inorganic_classification import (
    WikipediaOrganicInorganicClassification,
)
from .wikipedia_salts_semiconductors_classification import (
    WikipediaSaltsSemiconductorsClassification,
)
from .wikipedia_solid_state_colloidal_classification import (
    WikipediaSolidStateColloidalClassification,
)
from .wikipedia_theoretical_applied_classification import (
    WikipediaTheoreticalAppliedClassification,
    WikipediaTheoreticalAppliedClassificationV2,
)
from .yahoo_answers_topics_classification import (
    YahooAnswersTopicsClassification,
    YahooAnswersTopicsClassificationV2,
)
from .yelp_review_full_classification import (
    YelpReviewFullClassification,
    YelpReviewFullClassificationV2,
)

__all__ = [
    "AmazonPolarityClassification",
    "AmazonPolarityClassificationV2",
    "ArxivClassification",
    "ArxivClassificationV2",
    "Banking77Classification",
    "Banking77ClassificationV2",
    "BirdsnapClassification",
    "CIFAR10Classification",
    "CIFAR100Classification",
    "CUADAffiliateLicenseLicenseeLegalBenchClassification",
    "CUADAffiliateLicenseLicensorLegalBenchClassification",
    "CUADAntiAssignmentLegalBenchClassification",
    "CUADAuditRightsLegalBenchClassification",
    "CUADCapOnLiabilityLegalBenchClassification",
    "CUADChangeOfControlLegalBenchClassification",
    "CUADCompetitiveRestrictionExceptionLegalBenchClassification",
    "CUADCovenantNotToSueLegalBenchClassification",
    "CUADEffectiveDateLegalBenchClassification",
    "CUADExclusivityLegalBenchClassification",
    "CUADExpirationDateLegalBenchClassification",
    "CUADGoverningLawLegalBenchClassification",
    "CUADIPOwnershipAssignmentLegalBenchClassification",
    "CUADInsuranceLegalBenchClassification",
    "CUADIrrevocableOrPerpetualLicenseLegalBenchClassification",
    "CUADJointIPOwnershipLegalBenchClassification",
    "CUADLicenseGrantLegalBenchClassification",
    "CUADLiquidatedDamagesLegalBenchClassification",
    "CUADMinimumCommitmentLegalBenchClassification",
    "CUADMostFavoredNationLegalBenchClassification",
    "CUADNoSolicitOfCustomersLegalBenchClassification",
    "CUADNoSolicitOfEmployeesLegalBenchClassification",
    "CUADNonCompeteLegalBenchClassification",
    "CUADNonDisparagementLegalBenchClassification",
    "CUADNonTransferableLicenseLegalBenchClassification",
    "CUADNoticePeriodToTerminateRenewalLegalBenchClassification",
    "CUADPostTerminationServicesLegalBenchClassification",
    "CUADPriceRestrictionsLegalBenchClassification",
    "CUADRenewalTermLegalBenchClassification",
    "CUADRevenueProfitSharingLegalBenchClassification",
    "CUADRofrRofoRofnLegalBenchClassification",
    "CUADSourceCodeEscrowLegalBenchClassification",
    "CUADTerminationForConvenienceLegalBenchClassification",
    "CUADThirdPartyBeneficiaryLegalBenchClassification",
    "CUADUncappedLiabilityLegalBenchClassification",
    "CUADUnlimitedAllYouCanEatLicenseLegalBenchClassification",
    "CUADVolumeRestrictionLegalBenchClassification",
    "CUADWarrantyDurationLegalBenchClassification",
    "Caltech101Classification",
    "CanadaTaxCourtOutcomesLegalBenchClassification",
    "ContractNLIConfidentialityOfAgreementLegalBenchClassification",
    "ContractNLIExplicitIdentificationLegalBenchClassification",
    "ContractNLIInclusionOfVerballyConveyedInformationLegalBenchClassification",
    "ContractNLILimitedUseLegalBenchClassification",
    "ContractNLINoLicensingLegalBenchClassification",
    "ContractNLINoticeOnCompelledDisclosureLegalBenchClassification",
    "ContractNLIPermissibleAcquirementOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissibleCopyLegalBenchClassification",
    "ContractNLIPermissibleDevelopmentOfSimilarInformationLegalBenchClassification",
    "ContractNLIPermissiblePostAgreementPossessionLegalBenchClassification",
    "ContractNLIReturnOfConfidentialInformationLegalBenchClassification",
    "ContractNLISharingWithEmployeesLegalBenchClassification",
    "ContractNLISharingWithThirdPartiesLegalBenchClassification",
    "ContractNLISurvivalOfObligationsLegalBenchClassification",
    "CorporateLobbyingLegalBenchClassification",
    "Country211Classification",
    "DBpediaClassification",
    "DBpediaClassificationV2",
    "DTDClassification",
    "DefinitionClassificationLegalBenchClassification",
    "Diversity1LegalBenchClassification",
    "Diversity2LegalBenchClassification",
    "Diversity3LegalBenchClassification",
    "Diversity4LegalBenchClassification",
    "Diversity5LegalBenchClassification",
    "Diversity6LegalBenchClassification",
    "EmotionClassification",
    "EmotionClassificationV2",
    "EuroSATClassification",
    "FER2013Classification",
    "FGVCAircraftClassification",
    "FinancialPhrasebankClassification",
    "FinancialPhrasebankClassificationV2",
    "Food101Classification",
    "FrenkEnClassification",
    "FrenkEnClassificationV2",
    "FunctionOfDecisionSectionLegalBenchClassification",
    "GTSRBClassification",
    "HUMEEmotionClassification",
    "HUMEToxicConversationsClassification",
    "HUMETweetSentimentExtractionClassification",
    "Imagenet1kClassification",
    "ImdbClassification",
    "ImdbClassificationV2",
    "InsurancePolicyInterpretationLegalBenchClassification",
    "InternationalCitizenshipQuestionsLegalBenchClassification",
    "JCrewBlockerLegalBenchClassification",
    "JCrewBlockerLegalBenchClassificationV2",
    "LearnedHandsBenefitsLegalBenchClassification",
    "LearnedHandsBusinessLegalBenchClassification",
    "LearnedHandsConsumerLegalBenchClassification",
    "LearnedHandsCourtsLegalBenchClassification",
    "LearnedHandsCrimeLegalBenchClassification",
    "LearnedHandsDivorceLegalBenchClassification",
    "LearnedHandsDomesticViolenceLegalBenchClassification",
    "LearnedHandsEducationLegalBenchClassification",
    "LearnedHandsEmploymentLegalBenchClassification",
    "LearnedHandsEstatesLegalBenchClassification",
    "LearnedHandsFamilyLegalBenchClassification",
    "LearnedHandsHealthLegalBenchClassification",
    "LearnedHandsHousingLegalBenchClassification",
    "LearnedHandsImmigrationLegalBenchClassification",
    "LearnedHandsTortsLegalBenchClassification",
    "LearnedHandsTrafficLegalBenchClassification",
    "LegalReasoningCausalityLegalBenchClassification",
    "LegalReasoningCausalityLegalBenchClassificationV2",
    "MAUDLegalBenchClassification",
    "MAUDLegalBenchClassificationV2",
    "MNISTClassification",
    "NYSJudicialEthicsLegalBenchClassification",
    "NewsClassification",
    "NewsClassificationV2",
    "OPP115DataRetentionLegalBenchClassification",
    "OPP115DataSecurityLegalBenchClassification",
    "OPP115DataSecurityLegalBenchClassificationV2",
    "OPP115DoNotTrackLegalBenchClassification",
    "OPP115DoNotTrackLegalBenchClassificationV2",
    "OPP115FirstPartyCollectionUseLegalBenchClassification",
    "OPP115InternationalAndSpecificAudiencesLegalBenchClassification",
    "OPP115PolicyChangeLegalBenchClassification",
    "OPP115ThirdPartySharingCollectionLegalBenchClassification",
    "OPP115UserAccessEditAndDeletionLegalBenchClassification",
    "OPP115UserChoiceControlLegalBenchClassification",
    "OPP115UserChoiceControlLegalBenchClassificationV2",
    "OralArgumentQuestionPurposeLegalBenchClassification",
    "OralArgumentQuestionPurposeLegalBenchClassificationV2",
    "OverrulingLegalBenchClassification",
    "OverrulingLegalBenchClassificationV2",
    "OxfordFlowersClassification",
    "OxfordPetsClassification",
    "PROALegalBenchClassification",
    "PatchCamelyonClassification",
    "PatentClassification",
    "PatentClassificationV2",
    "PersonalJurisdictionLegalBenchClassification",
    "PoemSentimentClassification",
    "PoemSentimentClassificationV2",
    "RESISC45Classification",
    "SCDBPAccountabilityLegalBenchClassification",
    "SCDBPAuditsLegalBenchClassification",
    "SCDBPCertificationLegalBenchClassification",
    "SCDBPTrainingLegalBenchClassification",
    "SCDBPVerificationLegalBenchClassification",
    "SCDDAccountabilityLegalBenchClassification",
    "SCDDAuditsLegalBenchClassification",
    "SCDDCertificationLegalBenchClassification",
    "SCDDTrainingLegalBenchClassification",
    "SCDDVerificationLegalBenchClassification",
    "SDSEyeProtectionClassification",
    "SDSEyeProtectionClassificationV2",
    "SDSGlovesClassification",
    "SDSGlovesClassificationV2",
    "STL10Classification",
    "SUN397Classification",
    "StanfordCarsClassification",
    "TelemarketingSalesRuleLegalBenchClassification",
    "TextualismToolDictionariesLegalBenchClassification",
    "TextualismToolPlainLegalBenchClassification",
    "ToxicChatClassification",
    "ToxicChatClassificationV2",
    "ToxicConversationsClassification",
    "ToxicConversationsClassificationV2",
    "TweetSentimentExtractionClassification",
    "TweetSentimentExtractionClassificationV2",
    "TweetTopicSingleClassification",
    "TweetTopicSingleClassificationV2",
    "UCCVCommonLawLegalBenchClassification",
    "UCF101Classification",
    "UnfairTOSLegalBenchClassification",
    "WikipediaBioMetChemClassification",
    "WikipediaBioMetChemClassificationV2",
    "WikipediaBiolumNeurochemClassification",
    "WikipediaChemEngSpecialtiesClassification",
    "WikipediaChemFieldsClassification",
    "WikipediaChemFieldsClassificationV2",
    "WikipediaChemistryTopicsClassification",
    "WikipediaCompChemSpectroscopyClassification",
    "WikipediaCompChemSpectroscopyClassificationV2",
    "WikipediaCryobiologySeparationClassification",
    "WikipediaCrystallographyAnalyticalClassification",
    "WikipediaCrystallographyAnalyticalClassificationV2",
    "WikipediaGreenhouseEnantiopureClassification",
    "WikipediaIsotopesFissionClassification",
    "WikipediaLuminescenceClassification",
    "WikipediaOrganicInorganicClassification",
    "WikipediaSaltsSemiconductorsClassification",
    "WikipediaSolidStateColloidalClassification",
    "WikipediaTheoreticalAppliedClassification",
    "WikipediaTheoreticalAppliedClassificationV2",
    "YahooAnswersTopicsClassification",
    "YahooAnswersTopicsClassificationV2",
    "YelpReviewFullClassification",
    "YelpReviewFullClassificationV2",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class TempReasonL3Context(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="TempReasonL3Context",
        description="Measuring the ability to retrieve the groundtruth answers to reasoning task queries on TempReason l3-context.",
        reference="https://github.com/DAMO-NLP-SG/TempReason",
        dataset={
            "path": "mteb/TempReasonL3Context",
            "revision": "2c341c955bf2dc4f229c54b5b2cb9c5f1409ffd7",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2023-01-01", "2023-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Reasoning as Retrieval"],
        license="cc-by-sa-3.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{tan2023towards,
  author = {Tan, Qingyu and Ng, Hwee Tou and Bing, Lidong},
  journal = {arXiv preprint arXiv:2306.08952},
  title = {Towards benchmarking and improving the temporal reasoning capability of large language models},
  year = {2023},
}

@article{xiao2024rar,
  author = {Xiao, Chenghao and Hudson, G Thomas and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2404.06347},
  title = {RAR-b: Reasoning as Retrieval Benchmark},
  year = {2024},
}
""",
        prompt={
            "query": "Given the following question, facts and contexts, retrieve the correct answer."
        },
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class STS16STS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="STS16",
        dataset={
            "path": "mteb/sts16-sts",
            "revision": "4d8694f8f0e0100860b497b999b3dbed754a0513",
        },
        description="SemEval-2016 Task 4",
        reference="https://www.aclweb.org/anthology/S16-1001",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cosine_spearman",
        date=("2015-10-01", "2015-12-31"),
        domains=["Blog", "Web", "Spoken"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{nakov-etal-2016-semeval,
  address = {San Diego, California},
  author = {Nakov, Preslav  and
Ritter, Alan  and
Rosenthal, Sara  and
Sebastiani, Fabrizio  and
Stoyanov, Veselin},
  booktitle = {Proceedings of the 10th International Workshop on Semantic Evaluation ({S}em{E}val-2016)},
  doi = {10.18653/v1/S16-1001},
  editor = {Bethard, Steven  and
Carpuat, Marine  and
Cer, Daniel  and
Jurgens, David  and
Nakov, Preslav  and
Zesch, Torsten},
  month = jun,
  pages = {1--18},
  publisher = {Association for Computational Linguistics},
  title = {{S}em{E}val-2016 Task 4: Sentiment Analysis in {T}witter},
  url = {https://aclanthology.org/S16-1001},
  year = {2016},
}
""",
    )

    min_score = 0
    max_score = 5



from collections import defaultdict

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BLINKIT2TMultiChoice(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="BLINKIT2TMultiChoice",
        description="Retrieve the correct text answer based on images and specific retrieval instructions.",
        reference="https://arxiv.org/abs/2404.12390",
        dataset={
            "path": "JamieSJS/blink-it2t-multi",
            "revision": "bc8f4c7f62450a4ceb737c8339061cf87aea42d5",
        },
        type="VisionCentricQA",
        category="it2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2018-01-01", "2018-12-31"),
        domains=["Encyclopaedic"],
        task_subtypes=["Image Text Retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@article{fu2024blink,
  author = {Fu, Xingyu and Hu, Yushi and Li, Bangzheng and Feng, Yu and Wang, Haoyu and Lin, Xudong and Roth, Dan and Smith, Noah A and Ma, Wei-Chiu and Krishna, Ranjay},
  journal = {arXiv preprint arXiv:2404.12390},
  title = {Blink: Multimodal large language models can see but not perceive},
  year = {2024},
}
""",
    )

    def dataset_transform(self, **kwargs):
        for subset, split_data in self.dataset.items():
            for split, dataset in split_data.items():
                top_ranked = defaultdict(list)
                for query_id, relevant in dataset["relevant_docs"].items():
                    for corpus_id, score in relevant.items():
                        top_ranked[query_id].append(corpus_id)
                dataset["top_ranked"] = top_ranked



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class FiQAPLRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="FiQA-PL",
        description="Financial Opinion Mining and Question Answering",
        reference="https://sites.google.com/view/fiqa/",
        dataset={
            "path": "mteb/FiQA-PL",
            "revision": "788508256800f93c2ad033e96243bc801f9ee525",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=["Written", "Financial"],
        task_subtypes=["Question answering"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{thakur2021beir,
  author = {Nandan Thakur and Nils Reimers and Andreas R{\"u}ckl{\'e} and Abhishek Srivastava and Iryna Gurevych},
  booktitle = {Thirty-fifth Conference on Neural Information Processing Systems Datasets and Benchmarks Track (Round 2)},
  title = {{BEIR}: A Heterogeneous Benchmark for Zero-shot Evaluation of Information Retrieval Models},
  url = {https://openreview.net/forum?id=wCu6T5xFjeJ},
  year = {2021},
}
""",
        adapted_from=["FiQA2018"],
    )



from mteb.abstasks.aggregate_task_metadata import AggregateTaskMetadata
from mteb.abstasks.aggregated_task import AbsTaskAggregate
from mteb.tasks.sts.multilingual.sts17_multilingual_visual_sts import (
    STS17MultilingualVisualSTS,
)

task_list_sts17_multi = [
    STS17MultilingualVisualSTS().filter_languages(
        languages=["ara", "eng", "spa", "kor"],
        hf_subsets=[
            "ko-ko",
            "ar-ar",
            "en-ar",
            "en-de",
            "en-tr",
            "es-en",
            "es-es",
            "fr-en",
            "it-en",
            "nl-en",
        ],
    )
]


class STS17MultilingualVisualSTSMultilingual(AbsTaskAggregate):
    metadata = AggregateTaskMetadata(
        name="VisualSTS17Multilingual",
        description="STS17MultilingualVisualSTS multilingual.",
        reference="https://arxiv.org/abs/2402.08183/",
        tasks=task_list_sts17_multi,
        category="i2i",
        modalities=["image"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[""],
        sample_creation="rendered",
        main_score="cosine_spearman",
        type="VisualSTS(multi)",
        eval_splits=["test"],
        eval_langs={  # rely on subsets to filter scores in TaskResults.get_score_fast().
            "ko-ko": ["kor-Hang"],
            "ar-ar": ["ara-Arab"],
            "en-ar": ["eng-Latn", "ara-Arab"],
            "en-de": ["eng-Latn", "deu-Latn"],
            "en-tr": ["eng-Latn", "tur-Latn"],
            "es-en": ["spa-Latn", "eng-Latn"],
            "es-es": ["spa-Latn"],
            "fr-en": ["fra-Latn", "eng-Latn"],
            "it-en": ["ita-Latn", "eng-Latn"],
            "nl-en": ["nld-Latn", "eng-Latn"],
        },
        bibtex_citation=r"""
@article{xiao2024pixel,
  author = {Xiao, Chenghao and Huang, Zhuoxu and Chen, Danlu and Hudson, G Thomas and Li, Yizhi and Duan, Haoran and Lin, Chenghua and Fu, Jie and Han, Jungong and Moubayed, Noura Al},
  journal = {arXiv preprint arXiv:2402.08183},
  title = {Pixel Sentence Representation Learning},
  year = {2024},
}
""",
    )



from .dado_eval_coarse_classification import DadoEvalCoarseClassification
from .ita_casehold_classification import ItaCaseholdClassification
from .italian_linguist_acceptability_classification import (
    ItalianLinguisticAcceptabilityClassification,
    ItalianLinguisticAcceptabilityClassificationV2,
)
from .sardi_stance_classification import SardiStanceClassification

__all__ = [
    "DadoEvalCoarseClassification",
    "ItaCaseholdClassification",
    "ItalianLinguisticAcceptabilityClassification",
    "ItalianLinguisticAcceptabilityClassificationV2",
    "SardiStanceClassification",
]



from __future__ import annotations

from typing import TYPE_CHECKING

import torch

from mteb.models.model_meta import ScoringFunction

if TYPE_CHECKING:
    from mteb.models import EncoderProtocol
    from mteb.types import Array


def _use_torch_compile():
    gpu_ok = False
    if torch.cuda.is_available():
        device_cap = torch.cuda.get_device_capability()
        if device_cap in ((7, 0), (8, 0), (9, 0)):
            gpu_ok = True

    return gpu_ok


def _convert_to_tensor(a: Array, dtype=torch.float32) -> torch.Tensor:
    if not isinstance(a, torch.Tensor):
        a = torch.tensor(a, dtype=dtype)
    return a


def compute_pairwise_similarity(
    model: EncoderProtocol, embedding1: Array, embedding2: Array
) -> Array:
    """Compute pairwise similarity between two sets of embeddings using the model's built-in similarity function if available, otherwise using cosine similarity.

    Args:
        model: An instance of EncoderProtocol which may have a custom similarity function.
        embedding1: The first set of embeddings.
        embedding2: The second set of embeddings.

    Returns:
        Array: The computed pairwise similarity scores.
    """
    if hasattr(model, "similarity_pairwise"):
        return model.similarity_pairwise(embedding1, embedding2)
    return pairwise_cos_sim(embedding1, embedding2)


def select_similarity(
    embedding1: Array,
    embedding2: Array,
    similarity_fn: ScoringFunction,
) -> Array:
    """Compute similarity between two sets of embeddings using the specified similarity function.

    Args:
        embedding1: The first set of embeddings.
        embedding2: The second set of embeddings.
        similarity_fn: The similarity function to use (COSINE, DOT_PRODUCT, EUCLIDEAN).

    Returns:
        Array: The computed similarity scores.
    """
    if similarity_fn is ScoringFunction.COSINE:
        return cos_sim(embedding1, embedding2)
    elif similarity_fn is ScoringFunction.DOT_PRODUCT:
        return dot_score(embedding1, embedding2)
    elif similarity_fn is ScoringFunction.EUCLIDEAN:
        return euclidean_sim(embedding1, embedding2)
    raise ValueError(f"Unsupported similarity function: {similarity_fn}")


def select_pairwise_similarity(
    embedding1: Array,
    embedding2: Array,
    similarity_fn: ScoringFunction,
) -> Array:
    """Compute pairwise similarity between two sets of embeddings using the specified similarity function.

    Args:
        embedding1: The first set of embeddings.
        embedding2: The second set of embeddings.
        similarity_fn: The similarity function to use (COSINE, DOT_PRODUCT, EUCLIDEAN).

    Returns:
        Array: The computed pairwise similarity scores.
    """
    if similarity_fn is ScoringFunction.COSINE:
        return pairwise_cos_sim(embedding1, embedding2)
    elif similarity_fn is ScoringFunction.DOT_PRODUCT:
        return pairwise_dot_score(embedding1, embedding2)
    elif similarity_fn is ScoringFunction.EUCLIDEAN:
        return pairwise_euclidean_sim(embedding1, embedding2)
    raise ValueError(f"Unsupported similarity function: {similarity_fn}")


def _normalize_embeddings(embeddings: Array) -> torch.Tensor:
    """Normalizes the embeddings matrix, so that each sentence embedding has unit length.

    Args:
        embeddings: The input embeddings matrix.

    Returns:
        Tensor: The normalized embeddings matrix.
    """
    embeddings = _convert_to_tensor(embeddings)
    return torch.nn.functional.normalize(embeddings, p=2, dim=1)


def cos_sim(a: Array, b: Array) -> torch.Tensor:
    """Calculate pairwise cosine similarities between two sets of vectors.

    Computes the cosine similarity cos_sim(a[i], b[j]) for all i and j.

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Matrix with res[i][j]  = cos_sim(a[i], b[j])
    """
    # Move tensor conversion outside the compiled function
    # since compile works better with pure tensor operations
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)

    # The actual function to compile
    def _cos_sim_core(a_tensor, b_tensor):
        if len(a_tensor.shape) == 1:
            a_tensor = a_tensor.reshape(1, *a_tensor.shape)
        if len(b_tensor.shape) == 1:
            b_tensor = b_tensor.reshape(1, *b_tensor.shape)

        a_norm = _normalize_embeddings(a_tensor)
        b_norm = _normalize_embeddings(b_tensor)
        return a_norm @ b_norm.transpose(0, 1)

    # Compile the core function once
    should_compile = (
        hasattr(torch, "compile")
        and _use_torch_compile()
        and (isinstance(a, torch.Tensor) and isinstance(b, torch.Tensor))
    )
    if should_compile:
        _cos_sim_core_compiled = torch.compile(_cos_sim_core)
        return _cos_sim_core_compiled(a, b)
    else:
        return _cos_sim_core(a, b)


# https://github.com/UKPLab/sentence-transformers/blob/3fd59c3d122f2148e22b6338447b45d850fb6ea4/sentence_transformers/util.py#L125
def pairwise_cos_sim(a: Array, b: Array) -> Array:
    """Computes the pairwise cosine similarity cos_sim(a[i], b[i]).

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Tensor: Vector with res[i] = cos_sim(a[i], b[i])
    """
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)
    return pairwise_dot_score(_normalize_embeddings(a), _normalize_embeddings(b))


def max_sim(a: Array, b: Array) -> torch.Tensor:
    """Compute the maximum pairwise similarity between tokens.

    Given two tensors `a` and `b` of shape (batch_size, num_tokens, token_dim),
    this function computes the maximum similarity `max_sim(a[i], b[j])` for all
    pairs of tokens `i` and `j` across the two inputs.

    Args:
        a: Tensor of shape (batch_size, num_tokens, token_dim).
        b: Tensor of shape (batch_size, num_tokens, token_dim).

    Returns:
        A tensor containing the maximum similarity values for each batch.
    """
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)

    if len(a.shape) == 2:
        a = a.reshape(1, *a.shape)  # eq. to a.unsqueeze(0)

    if len(b.shape) == 2:
        b = b.reshape(1, *b.shape)

    scores = torch.einsum(
        "ash,bth->abst",
        a,
        b,
    )

    return scores.max(axis=-1).values.sum(axis=-1)  # type: ignore[call-overload]


# https://github.com/lightonai/pylate/blob/2d094a724866d6e15701781528368438081c0157/pylate/scores/scores.py#L67C1-L122C38
def pairwise_max_sim(
    queries_embeddings: Array,
    documents_embeddings: Array,
) -> torch.Tensor:
    """Computes the ColBERT score for each query-document pair. The score is computed as the sum of maximum similarities between the query and the document for corresponding pairs.

    Args:
        queries_embeddings: The first tensor. The queries embeddings. Shape: (batch_size, num tokens queries, embedding_size)
        documents_embeddings: The second tensor. The documents embeddings. Shape: (batch_size, num tokens documents, embedding_size)

    Returns:
        Tensor: Vector with res[i] = max_sim(queries_embeddings[i], documents_embeddings[i])
    """
    scores = []

    for query_embedding, document_embedding in zip(
        queries_embeddings, documents_embeddings
    ):
        query_embedding = _convert_to_tensor(query_embedding)
        document_embedding = _convert_to_tensor(document_embedding)

        query_document_score = torch.einsum(
            "sh,th->st",
            query_embedding,
            document_embedding,
        )

        scores.append(query_document_score.max(axis=-1).values.sum())  # type: ignore[call-overload]

    return torch.stack(scores, dim=0)


def dot_score(a: Array, b: Array) -> torch.Tensor:
    """Calculate pairwise dot products between two sets of vectors.

    Computes the dot product dot_prod(a[i], b[j]) for all i and j.

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Matrix with res[i][j]  = dot_prod(a[i], b[j])
    """
    # Move tensor conversion outside the compiled function
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)

    # The actual function to compile
    def _dot_score_core(a_tensor, b_tensor):
        if len(a_tensor.shape) == 1:
            a_tensor = a_tensor.unsqueeze(0)
        if len(b_tensor.shape) == 1:
            b_tensor = b_tensor.unsqueeze(0)

        return a_tensor @ b_tensor.transpose(0, 1)

    # Compile the core function once
    if (
        hasattr(torch, "compile")
        and _use_torch_compile()
        and isinstance(a, torch.Tensor)
    ):
        _dot_score_core_compiled = torch.compile(_dot_score_core)
        return _dot_score_core_compiled(a, b)
    else:
        return _dot_score_core(a, b)


def pairwise_dot_score(a: Array, b: Array) -> Array:
    """Computes the pairwise dot-product dot_prod(a[i], b[i]).

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Tensor: Vector with res[i] = dot_prod(a[i], b[i])
    """
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)
    return (a * b).sum(dim=-1)


# https://github.com/UKPLab/sentence-transformers/blob/3fd59c3d122f2148e22b6338447b45d850fb6ea4/sentence_transformers/util.py#L196C1-L227C56
def euclidean_sim(a: Array, b: Array) -> Array:
    """Computes the euclidean similarity (i.e., negative distance) between two tensors.

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Tensor: Matrix with res[i][j] = -euclidean_distance(a[i], b[j])
    """
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)

    return -torch.cdist(a, b, p=2.0)


def pairwise_euclidean_sim(a: Array, b: Array) -> Array:
    """Computes the euclidean distance (i.e., negative distance) between pairs of tensors.

    Args:
        a: The first tensor.
        b: The second tensor.

    Returns:
        Vector with res[i] = -euclidean_distance(a[i], b[i])
    """
    a = _convert_to_tensor(a)
    b = _convert_to_tensor(b)

    return -torch.sqrt(torch.sum((a - b) ** 2, dim=-1))


def similarity(text_embeddings: Array, input_embeddings: Array) -> Array:
    """Similarity function used in ImageTextPair classification

    Args:
        text_embeddings: Embeddings of the text inputs
        input_embeddings: Embeddings of the image inputs

    Returns:
        Matrix with similarities
    """
    text_embeddings_tensor = _convert_to_tensor(text_embeddings)
    input_embeddings_tensor = _convert_to_tensor(input_embeddings)

    text_embeddings_tensor = text_embeddings_tensor / text_embeddings_tensor.norm(
        dim=-1, keepdim=True
    )
    input_embeddings_tensor = input_embeddings_tensor / input_embeddings_tensor.norm(
        dim=-1, keepdim=True
    )
    logits = torch.matmul(input_embeddings_tensor, text_embeddings_tensor.T)
    probs = (logits * 100).softmax(dim=-1)
    return probs



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BIRCORelicReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="BIRCO-Relic",
        description=(
            "Retrieval task using the RELIC dataset from BIRCO. This dataset contains 100 queries which are excerpts from literary analyses "
            "with a missing quotation (indicated by [masked sentence(s)]). Each query has a candidate pool of 50 passages. "
            "The objective is to retrieve the passage that best completes the literary analysis."
        ),
        reference="https://github.com/BIRCO-benchmark/BIRCO",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        dataset={
            "path": "mteb/BIRCO-Relic-Test",
            "revision": "3d593fde907860102a282b4a6ea7fb1e1f1f83dd",
        },
        date=("2024-01-01", "2024-12-31"),
        domains=["Fiction"],  # Valid domain
        task_subtypes=["Article retrieval"],  # Valid subtype
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        prompt="Given a literary analysis with a missing quotation (marked as [masked sentence(s)]), retrieve the passage that best completes the analysis.",
        bibtex_citation=r"""
@misc{wang2024bircobenchmarkinformationretrieval,
  archiveprefix = {arXiv},
  author = {Xiaoyue Wang and Jianyou Wang and Weili Cao and Kaicheng Wang and Ramamohan Paturi and Leon Bergen},
  eprint = {2402.14151},
  primaryclass = {cs.IR},
  title = {BIRCO: A Benchmark of Information Retrieval Tasks with Complex Objectives},
  url = {https://arxiv.org/abs/2402.14151},
  year = {2024},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SweRecClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SweRecClassification",
        description="A Swedish dataset for sentiment classification on review",
        reference="https://aclanthology.org/2023.nodalida-1.20/",
        dataset={
            "path": "mteb/swerec_classification",
            "revision": "b07c6ce548f6a7ac8d546e1bbe197a0086409190",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="accuracy",
        date=("2023-01-01", "2023-12-31"),  # based on the publication date
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{nielsen-2023-scandeval,
  address = {T{\'o}rshavn, Faroe Islands},
  author = {Nielsen, Dan},
  booktitle = {Proceedings of the 24th Nordic Conference on Computational Linguistics (NoDaLiDa)},
  editor = {Alum{\"a}e, Tanel  and
Fishel, Mark},
  month = may,
  pages = {185--201},
  publisher = {University of Tartu Library},
  title = {{S}cand{E}val: A Benchmark for {S}candinavian Natural Language Processing},
  url = {https://aclanthology.org/2023.nodalida-1.20},
  year = {2023},
}
""",
        prompt="Classify Swedish reviews by sentiment",
        superseded_by="SweRecClassification.v2",
    )


class SweRecClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SweRecClassification.v2",
        description="A Swedish dataset for sentiment classification on review This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://aclanthology.org/2023.nodalida-1.20/",
        dataset={
            "path": "mteb/swe_rec",
            "revision": "2a18a4ccc6770319b7f717cda1800f7d5bd5cd1a",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="accuracy",
        date=("2023-01-01", "2023-12-31"),  # based on the publication date
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{nielsen-2023-scandeval,
  address = {T{\'o}rshavn, Faroe Islands},
  author = {Nielsen, Dan},
  booktitle = {Proceedings of the 24th Nordic Conference on Computational Linguistics (NoDaLiDa)},
  editor = {Alum{\"a}e, Tanel  and
Fishel, Mark},
  month = may,
  pages = {185--201},
  publisher = {University of Tartu Library},
  title = {{S}cand{E}val: A Benchmark for {S}candinavian Natural Language Processing},
  url = {https://aclanthology.org/2023.nodalida-1.20},
  year = {2023},
}
""",
        prompt="Classify Swedish reviews by sentiment",
        adapted_from=["SweRecClassification"],
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class STS14STS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="STS14",
        dataset={
            "path": "mteb/sts14-sts",
            "revision": "6031580fec1f6af667f0bd2da0a551cf4f0b2375",
        },
        description="SemEval STS 2014 dataset. Currently only the English dataset",
        reference="https://www.aclweb.org/anthology/S14-1002",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cosine_spearman",
        date=("2012-01-01", "2012-08-31"),
        domains=["Blog", "Web", "Spoken"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{bandhakavi-etal-2014-generating,
  address = {Dublin, Ireland},
  author = {Bandhakavi, Anil  and
Wiratunga, Nirmalie  and
P, Deepak  and
Massie, Stewart},
  booktitle = {Proceedings of the Third Joint Conference on Lexical and Computational Semantics (*{SEM} 2014)},
  doi = {10.3115/v1/S14-1002},
  editor = {Bos, Johan  and
Frank, Anette  and
Navigli, Roberto},
  month = aug,
  pages = {12--21},
  publisher = {Association for Computational Linguistics and Dublin City University},
  title = {Generating a Word-Emotion Lexicon from {\#}Emotional Tweets},
  url = {https://aclanthology.org/S14-1002},
  year = {2014},
}
""",
    )

    min_score = 0
    max_score = 5
